import logging
import string

from django.conf import settings
from django.contrib import messages
from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
from django.http import Http404
from django.shortcuts import get_object_or_404, render
from django.utils.translation import gettext_lazy as _

from oldp.apps.cases.models import Case
from oldp.apps.laws.models import Law, LawBook
from oldp.utils.cache_per_user import cache_per_role

logger = logging.getLogger(__name__)


@cache_per_role(settings.CACHE_TTL)
def view_index(request, char=None):
    page = request.GET.get("page")
    items = LawBook.objects.filter(
        latest=True
    )  # .values('slug', 'title', 'code').annotate(n=models.Count('slug'))

    if char is not None and len(char) == 1:
        char = str(char).lower()
        items = items.filter(slug__startswith=char)  # Case sensitive filtering

        top_items = []
    else:
        items = items.all()
        top_items = items.order_by("-order")[:5]

    items = items.order_by("title")

    paginator = Paginator(items, settings.PAGINATE_BY)

    try:
        items = paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        items = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        items = paginator.page(paginator.num_pages)

    return render(
        request,
        "laws/index.html",
        {
            "nav": "laws",
            "items": items,
            "top_items": top_items,
            "char": char,
            "chars": list(string.ascii_lowercase),
            "title": _("Laws"),
        },
    )


def get_latest_law_book(book_slug):
    """Law book by slug and latest=true (logs warning if multiple instances exist)"""
    candidates = LawBook.objects.filter(slug=book_slug, latest=True)
    book = candidates.first()

    if book is None:
        logger.info("Law book not found: %s", book_slug)
        raise Http404()

    if candidates.count() > 1:
        logger.warning(
            "Book has more than one instance with latest=true: %s", book_slug
        )

    return book


def get_law_book(request, book_slug):
    """Law book by slug and optional revision_date"""
    revision_date = request.GET.get("revision_date")

    if revision_date:
        try:
            return LawBook.objects.get(slug=book_slug, revision_date=revision_date)
        except LawBook.DoesNotExist:
            logger.debug(
                "Requested revision not found: book=%s, revision_date=%s",
                book_slug,
                revision_date,
            )
            messages.warning(
                request,
                _(
                    "The requested revision (%s) was not found. Showing instead the latest revision."
                    % revision_date
                ),
            )
            return get_latest_law_book(book_slug)
    else:
        return get_latest_law_book(book_slug)


@cache_per_role(settings.CACHE_TTL)
def view_book(request, book_slug):
    book = get_law_book(request, book_slug)
    section_titles = book.get_sections()
    revision_dates = list(book.get_revision_dates())

    items_qs = (
        Law.objects.filter(book=book)
        .select_related("book")
        .defer(*Law.defer_fields_list_view)
        .order_by("order")
    )
    items = list(items_qs)
    for item in items:
        item.display_section = section_titles.get(str(item.order))

    return render(
        request,
        "laws/book.html",
        {
            "items": items,
            "book": book,
            "revision_dates": revision_dates,
            "title": book.get_title(),
            "nav": "laws",
        },
    )


@cache_per_role(settings.CACHE_TTL)
def view_law(request, law_slug, book_slug):
    book = get_law_book(request, book_slug)
    item = get_object_or_404(
        Law.objects.select_related("book", "previous"), slug=law_slug, book=book
    )
    revision_dates = list(book.get_revision_dates())
    related_laws = item.get_related()

    referencing_cases = item.get_referencing_cases(
        Case.get_queryset(request).defer(*Case.defer_fields_list_view)
    )

    return render(
        request,
        "laws/law.html",
        {
            "nav": "laws",
            "item": item,
            "title": item.get_title(),
            "revision_dates": revision_dates,
            "related_laws": related_laws,
            "referencing_cases": referencing_cases,
        },
    )



import re

from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.db.models import Q
from django.utils.text import slugify

from oldp.apps.cases.models import Case
from oldp.apps.lib.markers import BaseMarker

ANNOTATION_VALUE_TYPE_STRING = "str"
ANNOTATION_VALUE_TYPE_INTEGER = "int"

color_re = re.compile("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$")
validate_color = RegexValidator(color_re, "Enter a valid color.", "invalid")


class AnnotationLabel(models.Model):
    """Label for annotations (e.g. title, ...)"""

    DEFAULT_COLOR = "#CCCCCC"

    name = models.CharField(
        max_length=100, help_text="Verbose name, e.g. This Awesome annotation"
    )
    slug = models.SlugField(
        max_length=100,
        help_text="Identifier, e.g. this-awesome-annotation",
        db_index=True,
        blank=True,
    )
    owner = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        # TODO nullable? For global labels
    )
    trusted = models.BooleanField(
        help_text="Trusted annotations are display by default in front end",
        default=False,
        db_index=True,
    )
    private = models.BooleanField(
        help_text="Private annotations are only visible to its author",
        default=False,
        db_index=True,
    )
    many_annotations_per_label = models.BooleanField(
        help_text="A content object can have more than one annotation per label",
        default=False,
    )
    use_marker = models.BooleanField(
        default=False,
        help_text="Marker annotations are extracted from the text content and have a position in the text",
    )
    annotation_value_type = models.CharField(
        choices=(
            (ANNOTATION_VALUE_TYPE_STRING, "String"),
            (ANNOTATION_VALUE_TYPE_INTEGER, "Integer"),
        ),
        default="str",
        max_length=5,
        help_text="Annotation values must be in this data type",
    )

    # belongs_to_type = models.CharField(
    #     choices=['case']
    # )
    color = models.CharField(
        # TODO Maybe color field? https://github.com/jaredly/django-colorfield/blob/master/colorfield/fields.py
        max_length=18,
        default=DEFAULT_COLOR,
        validators=[validate_color],
    )
    created_at = models.DateTimeField(
        auto_now_add=True, help_text="Entry is created at this date time"
    )
    updated_at = models.DateTimeField(
        auto_now=True, help_text="Date time of last change"
    )

    class Meta:
        verbose_name = "Label"
        db_table = "annotations_label"
        unique_together = (
            "slug",
            "owner",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.annotations = []

    def get_full_slug(self):
        return self.owner.username + "/" + self.slug

    def get_private(self):
        return self.private

    def get_owner(self):
        return self.owner

    def clean(self):
        if self.slug == "":
            self.slug = slugify(self.name)

        super().clean()

        if self.trusted and self.private:
            raise ValidationError(
                {"trusted": "Label cannot be `trusted` and `private` at the same time."}
            )

    def __repr__(self):
        return self.name

    def __str__(self):
        return "<Label(#%i, %s, %s)>" % (self.pk, self.slug, self.owner.username)


class Annotation(models.Model):
    """An annotation assigns a value (int or string) to a single content item, e.g. case.
    The meaning of the value is defined by the corresponding label.
    """

    belongs_to = None
    label = models.ForeignKey(AnnotationLabel, on_delete=models.CASCADE)
    value_str = models.CharField(
        max_length=250,
        null=True,
        blank=True,
    )
    value_int = models.IntegerField(
        null=True,
        blank=True,
    )
    created_at = models.DateTimeField(
        auto_now_add=True, help_text="Entry is created at this date time"
    )
    updated_at = models.DateTimeField(
        auto_now=True, help_text="Date time of last change"
    )

    class Meta:
        abstract = True

    def value(self):
        if self.value_str is not None:
            return self.value_str
        elif self.value_int is not None:
            return self.value_int
        else:
            return None

    def get_private(self):
        return self.label.private

    def get_owner(self):
        return self.label.owner

    def clean(self):
        super().clean()
        # Check `many_to_annotations_per_label` constraint
        if self.label.many_annotations_per_label is False:
            # Retrieve ids for a useful error message
            other_ids = list(
                self.__class__._default_manager.filter(
                    label=self.label, belongs_to=self.belongs_to
                )
                .exclude(pk=self.pk)
                .values_list("pk", flat=True)
            )

            if len(other_ids) > 0:
                # TODO Returning the IDs as extra data field is currently not supoorted by DRF
                raise ValidationError(
                    {
                        "label": "Label does not allow multiple annotations for the same content object. "
                        "Other annotation IDs: {}".format(other_ids),
                    }
                )

        # Check on values
        if (
            self.label.annotation_value_type == ANNOTATION_VALUE_TYPE_STRING
            and self.value_str is None
        ):
            raise ValidationError(
                {
                    "value_str": "value_str cannot be null.",
                }
            )
        if (
            self.label.annotation_value_type == ANNOTATION_VALUE_TYPE_INTEGER
            and self.value_int is None
        ):
            raise ValidationError(
                {
                    "value_int": "value_int cannot be null.",
                }
            )

    def __repr__(self):
        return self.label.slug + "=%s" % self.value()

    def __str__(self):
        return "<Annotation(#%i, %s, %s, %s)>" % (
            self.pk,
            self.label.slug,
            self.belongs_to,
            self.value(),
        )


class Marker(Annotation, BaseMarker):
    """Similar to an annotation, but linked to the textual content of an item, e.g. highlighted text or a citation.
    Start and end define the position of the value within the textual content.
    """

    start = models.PositiveIntegerField(
        db_index=True, default=0, help_text="Start position of marker (from 0)"
    )
    end = models.PositiveIntegerField(
        db_index=True,
        default=0,
        help_text="End position of marker",
    )

    def get_position(self):
        return self.start, self.end

    class Meta:
        abstract = True

    def get_start_position(self) -> int:
        return self.start

    def get_end_position(self) -> int:
        return self.end

    def get_marker_open(self) -> str:
        return '<span id="marker{}" class="marker marker-label{}" style="background-color: {}">'.format(
            self.pk, self.label_id, self.label.color
        )

    def get_marker_close(self) -> str:
        return '<span class="marker-label">{}</span></span>'.format(self.label.name)

    def get_marker_close_format(self):
        # Do not use formatting
        pass

    def get_marker_open_format(self):
        pass

    def clean(self):
        super().clean()

        # Check for valid position: start <= end
        # - Markers with zero-length are allowed, hence, less or equal not only strict less.
        if self.start > self.end:
            raise ValidationError(
                {"start": "Invalid marker position. Start cannot be greater than end."}
            )

        # Check if marker overlaps with other markers
        other_ids = list(
            self.__class__._default_manager.filter(
                Q(label=self.label),
                Q(belongs_to=self.belongs_to),
                Q(
                    Q(
                        start__gte=self.start, start__lte=self.end
                    )  # Other start is in range
                    | Q(end__gt=self.start, end__lt=self.end)  # Other end is in range
                    | Q(
                        start__lte=self.start, end__gt=self.start
                    )  # Start is in range of others
                    | Q(
                        start__lte=self.end, end__gt=self.end
                    )  # End is in range of others
                ),
            )
            .exclude(pk=self.pk)
            .values_list("pk", flat=True)
        )
        if len(other_ids) > 0:
            raise ValidationError(
                {
                    "start": "Marker overlaps with other existing markers. Other marker IDs: %s"
                    % other_ids,
                }
            )

    def __str__(self):
        return "<Marker(#%i, %s, %s, %s)>" % (
            self.pk,
            self.label.slug,
            self.belongs_to,
            self.value(),
        )


# Content implementation (each content class should have its own implementation)
class CaseAnnotation(Annotation):
    belongs_to = models.ForeignKey(Case, on_delete=models.CASCADE)


class CaseMarker(Marker):
    belongs_to = models.ForeignKey(Case, on_delete=models.CASCADE)



"""Court resolver service for resolving court from name/string input.

Extracted from assign_court processing step for reuse in API.
"""

import logging
import re
from typing import Optional, Tuple

from django.conf import settings

from oldp.apps.cases.exceptions import CourtNotFoundError
from oldp.apps.courts.apps import CourtLocationLevel
from oldp.apps.courts.models import City, Court, State
from oldp.utils import find_from_mapping

logger = logging.getLogger(__name__)


class CourtResolver:
    """Service to resolve court from name/string input.

    This service extracts the court resolution logic from the assign_court
    processing step to allow reuse in the case creation API.
    """

    def remove_chamber(self, name: str) -> Tuple[str, Optional[str]]:
        """Extract chamber designation from court name.

        Examples:
            - "LG Kiel Kammer für Handelssachen" -> ("LG Kiel", "Kammer für Handelssachen")
            - "LG Koblenz 14. Zivilkammer" -> ("LG Koblenz", "14. Zivilkammer")
            - "OLG Koblenz 2. Senat für Bußgeldsachen" -> ("OLG Koblenz", "2. Senat für Bußgeldsachen")

        Args:
            name: Court name potentially containing chamber designation

        Returns:
            Tuple of (court_name_without_chamber, chamber_designation or None)
        """
        chamber = None
        patterns = [
            r"\s([0-9]+)(.*)$",
            r"\s(Senat|Kammer) für(.*)$",
            r"\s([a-zA-Z]+)(senat|kammer)(.*)$",
        ]

        for pattern in patterns:
            compiled_pattern = re.compile(pattern)
            match = re.search(compiled_pattern, name)
            if match:
                name = name[: match.start()] + name[match.end() :]
                chamber = match.group(0).strip()

        return name.strip(), chamber

    def find_court(self, court_name: str, court_code: Optional[str] = None) -> Court:
        """Find court by name, code, or alias.

        Resolution order:
        1. By code (if provided)
        2. By exact name match (if name has no spaces)
        3. By court type + state location
        4. By court type + city location
        5. By alias (case-insensitive)

        Args:
            court_name: Court name to search for
            court_code: Optional court code (e.g., "EuGH", "BGH")

        Returns:
            Court instance

        Raises:
            CourtNotFoundError: If court cannot be resolved
        """
        # Try to find by code first
        if court_code:
            try:
                return Court.objects.get(code=court_code)
            except Court.DoesNotExist:
                pass

        if not court_name:
            raise CourtNotFoundError("Court name is required")

        # Handle special case for EU court
        if court_name == "EU":
            try:
                return Court.objects.get(code="EuGH")
            except Court.DoesNotExist:
                pass

        # Try exact name match first
        try:
            return Court.objects.get(name=court_name)
        except Court.DoesNotExist:
            pass

        # Try matching by court code (e.g. "BVerfG", "BGH")
        try:
            return Court.objects.get(code=court_name)
        except Court.DoesNotExist:
            pass

        # Determine court type
        court_type = Court.extract_type_code_from_name(court_name)

        if court_type is None:
            # Try alias search as fallback
            court = self._find_by_alias(court_name)
            if court:
                return court
            raise CourtNotFoundError(
                f"Could not determine court type from name: {court_name}"
            )

        try:
            location_levels = settings.COURT_TYPES.get_type(court_type)["levels"]
        except (KeyError, TypeError):
            raise CourtNotFoundError(f"Unknown court type: {court_type}")

        # Look for states
        if CourtLocationLevel.STATE in location_levels:
            court = self._find_by_state(court_name, court_type)
            if court:
                return court

        # Look for cities
        if CourtLocationLevel.CITY in location_levels:
            court = self._find_by_city(court_name, court_type)
            if court:
                return court

        # Search by alias as last resort
        court = self._find_by_alias(court_name)
        if court:
            return court

        # Try partial name match with court type filter (e.g. "VGH München" → type VGH, 1 match)
        if court_type:
            court = self._find_by_partial_name(court_name, court_type)
            if court:
                return court

        raise CourtNotFoundError(f"Could not resolve court from name: {court_name}")

    def _find_by_state(self, court_name: str, court_type: str) -> Optional[Court]:
        """Find court by state and type."""
        state_id_mapping = {}
        for state_id, state_name in State.objects.values_list("id", "name"):
            if state_name:
                state_id_mapping[state_name] = state_id
                # Add variations, e.g. Hamburg_er, Holstein_isches
                for variation in ["es", "er", "isches"]:
                    state_id_mapping[state_name + variation] = state_id

        state_id = find_from_mapping(court_name, state_id_mapping)

        if state_id is not None:
            try:
                logger.debug("Look for state=%i, type=%s", state_id, court_type)
                return Court.objects.get(state_id=state_id, court_type=court_type)
            except Court.DoesNotExist:
                pass

        return None

    def _find_by_city(self, court_name: str, court_type: str) -> Optional[Court]:
        """Find court by city and type."""
        city_id_mapping = {}
        for city_id, city_name in City.objects.values_list("id", "name"):
            if city_name:
                city_id_mapping[city_name] = city_id

        city_id = find_from_mapping(court_name, city_id_mapping)

        if city_id is not None:
            try:
                logger.debug("Look for city=%i, type=%s", city_id, court_type)
                return Court.objects.get(city_id=city_id, court_type=court_type)
            except Court.DoesNotExist:
                pass

        return None

    def _find_by_partial_name(
        self, court_name: str, court_type: str
    ) -> Optional[Court]:
        """Find court by extracting location words and matching against court names.

        For input like 'VGH München', extract 'München' and find courts of type VGH
        whose name contains 'München' or whose state contains a city named 'München'.
        Falls back to finding the court's state via City model.
        """
        # Extract the location part (everything after the court type prefix)
        words = court_name.split()
        location_parts = [w for w in words if w.upper() != court_type.upper()]
        if not location_parts:
            return None

        location = " ".join(location_parts)

        # Try direct name match: "VGH München" → courts of type VGH with "München" in name
        candidates = Court.objects.filter(
            court_type=court_type, name__icontains=location
        )
        if candidates.count() == 1:
            return candidates.first()

        # Try city → state resolution: München is in Bayern → find VGH in Bayern
        try:
            city = City.objects.get(name=location)
            if city.state_id:
                try:
                    return Court.objects.get(
                        court_type=court_type, state_id=city.state_id
                    )
                except Court.DoesNotExist:
                    pass
        except City.DoesNotExist:
            pass

        return None

    def _find_by_alias(self, court_name: str) -> Optional[Court]:
        """Find court by alias (case-insensitive).

        First tries icontains. If multiple candidates, narrows to exact line match.
        """
        candidates = Court.objects.filter(aliases__icontains=court_name)
        if len(candidates) == 1:
            return candidates.first()
        elif len(candidates) > 1:
            # Disambiguate: check for exact line match in aliases
            exact = [
                c
                for c in candidates
                if court_name.lower()
                in [a.strip().lower() for a in (c.aliases or "").splitlines()]
            ]
            if len(exact) == 1:
                return exact[0]
            logger.warning(
                "Multiple court candidates found for '%s': %s",
                court_name,
                [c.name for c in candidates],
            )
        return None

    def resolve(
        self, court_name: str, court_code: Optional[str] = None
    ) -> Tuple[Court, Optional[str]]:
        """Resolve court from name, extracting chamber if present.

        This is the main entry point for court resolution.

        Args:
            court_name: Court name (may include chamber designation)
            court_code: Optional court code

        Returns:
            Tuple of (Court instance, chamber designation or None)

        Raises:
            CourtNotFoundError: If court cannot be resolved
        """
        # Extract chamber from name
        clean_name, chamber = self.remove_chamber(court_name)

        # Find the court
        court = self.find_court(clean_name, court_code)

        return court, chamber



# Generated by Django 2.0.4 on 2018-04-25 09:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("search", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="searchquery",
            name="query",
            field=models.CharField(
                help_text="Query as entered by user but all lower case",
                max_length=200,
                unique=True,
            ),
        ),
        migrations.AddIndex(
            model_name="searchquery",
            index=models.Index(fields=["query"], name="search_sear_query_eedf16_idx"),
        ),
    ]



import logging

from django.contrib.auth.models import User
from django.core.management.base import BaseCommand, CommandError

from oldp.apps.accounts.models import (
    APIToken,
    APITokenPermission,
    APITokenPermissionGroup,
)
from oldp.apps.cases.models import Case
from oldp.apps.courts.models import Court

logger = logging.getLogger(__name__)

DUMMY_API_CASES = [
    {
        "title": "",
        "slug": "bgh-2024-03-15-i-zr-42-23",
        "court_code": "BGH",
        "court_chamber": None,
        "date": "2024-03-15",
        "file_number": "I ZR 42/23",
        "type": "Urteil",
        "ecli": "ECLI:DE:BGH:2024:150324UIZR42.23.0",
        "content": (
            "<h2>Tenor</h2>"
            "<p>Die Revision des Beklagten gegen das Urteil des 6. Zivilsenats "
            "des Oberlandesgerichts Köln vom 15. Dezember 2023 wird "
            "zurückgewiesen.</p>"
            "<h2>Gründe</h2>"
            "<p>I. Der Kläger nimmt den Beklagten auf Schadensersatz wegen "
            "Verletzung von Verkehrssicherungspflichten in Anspruch. "
            "Das Berufungsgericht hat der Klage stattgegeben. "
            "Hiergegen wendet sich die Revision des Beklagten.</p>"
            "<p>II. Die Revision ist unbegründet. Das Berufungsgericht hat "
            "zutreffend einen Anspruch des Klägers aus § 823 Abs. 1 BGB "
            "bejaht. Der Beklagte hat die ihm obliegenden "
            "Verkehrssicherungspflichten schuldhaft verletzt.</p>"
        ),
        "abstract": (
            "<p>Zur Haftung bei Verletzung von Verkehrssicherungspflichten "
            "im Zusammenhang mit der Instandhaltung von Gehwegen.</p>"
        ),
        "source_url": "https://api.example.com/cases/bgh-i-zr-42-23",
    },
    {
        "title": "",
        "slug": "bverfg-2024-06-20-1-bvr-567-23",
        "court_code": "BVerfG",
        "court_chamber": "1. Senat",
        "date": "2024-06-20",
        "file_number": "1 BvR 567/23",
        "type": "Beschluss",
        "ecli": "ECLI:DE:BVerfG:2024:rs20240620.1bvr056723",
        "content": (
            "<h2>Tenor</h2>"
            "<p>Die Verfassungsbeschwerde wird nicht zur Entscheidung "
            "angenommen.</p>"
            "<h2>Gründe</h2>"
            "<p>Die Verfassungsbeschwerde ist nicht zur Entscheidung "
            "anzunehmen, weil die Annahmevoraussetzungen des § 93a Abs. 2 "
            "BVerfGG nicht vorliegen. Die Verfassungsbeschwerde hat keine "
            "grundsätzliche verfassungsrechtliche Bedeutung im Sinne des "
            "Art. 14 Abs. 1 GG.</p>"
        ),
        "abstract": None,
        "source_url": "https://api.example.com/cases/bverfg-1-bvr-567-23",
    },
    {
        "title": "",
        "slug": "bgh-2024-09-10-viii-zr-88-24",
        "court_code": "BGH",
        "court_chamber": "VIII. Zivilsenat",
        "date": "2024-09-10",
        "file_number": "VIII ZR 88/24",
        "type": "Urteil",
        "ecli": "ECLI:DE:BGH:2024:100924UVIIIZR88.24.0",
        "content": (
            "<h2>Tenor</h2>"
            "<p>Auf die Revision der Klägerin wird das Urteil der "
            "8. Zivilkammer des Landgerichts München I vom 5. Mai 2024 "
            "aufgehoben.</p>"
            "<p>Die Sache wird zur neuen Verhandlung und Entscheidung, "
            "auch über die Kosten des Revisionsverfahrens, an das "
            "Berufungsgericht zurückverwiesen.</p>"
            "<h2>Gründe</h2>"
            "<p>I. Die Klägerin verlangt von dem Beklagten die Rückzahlung "
            "einer Mietkaution nach Beendigung des Mietverhältnisses.</p>"
            "<p>II. Das Berufungsgericht hat die Klage abgewiesen. "
            "Dies hält der rechtlichen Nachprüfung nicht stand.</p>"
            "<p>Nach § 551 Abs. 3 BGB hat der Vermieter die Kaution "
            "getrennt von seinem Vermögen anzulegen. Der Rückzahlungsanspruch "
            "ergibt sich aus § 812 Abs. 1 Satz 1 BGB.</p>"
        ),
        "abstract": (
            "<p>Zum Rückzahlungsanspruch des Mieters hinsichtlich der "
            "Mietkaution nach Beendigung des Mietverhältnisses.</p>"
        ),
        "source_url": "https://api.example.com/cases/bgh-viii-zr-88-24",
    },
]


class Command(BaseCommand):
    help = "Load dummy cases created via the API (with API token tracking)"

    def handle(self, *args, **options):
        if Case.objects.filter(
            created_by_token__isnull=False,
            slug__in=[c["slug"] for c in DUMMY_API_CASES],
        ).exists():
            raise CommandError(
                "Dummy API cases already exist. "
                "Delete them first if you want to reload."
            )

        # Get or create a user for the API token
        user, created = User.objects.get_or_create(
            username="api-scraper",
            defaults={"email": "scraper@example.com", "is_active": True},
        )
        if created:
            user.set_unusable_password()
            user.save()
            logger.info("Created api-scraper user")

        # Create permission: cases:write
        permission, _ = APITokenPermission.objects.get_or_create(
            resource="cases",
            action="write",
            defaults={"description": "Allows creating new cases via the API"},
        )

        # Create permission group
        group, group_created = APITokenPermissionGroup.objects.get_or_create(
            name="Case Writer",
            defaults={"description": "Permission group for submitting cases via API"},
        )
        if group_created:
            group.permissions.add(permission)
            logger.info("Created 'Case Writer' permission group")

        # Create API token
        token, token_created = APIToken.objects.get_or_create(
            user=user,
            name="Dummy Scraper Token",
            defaults={
                "is_active": True,
                "permission_group": group,
            },
        )
        if token_created:
            logger.info(
                "Created API token '%s' for user '%s'", token.name, user.username
            )

        # Resolve courts by code
        court_cache = {}
        for case_data in DUMMY_API_CASES:
            code = case_data["court_code"]
            if code not in court_cache:
                try:
                    court_cache[code] = Court.objects.get(code=code)
                except Court.DoesNotExist:
                    raise CommandError(
                        f"Court with code '{code}' not found. "
                        "Make sure court fixtures are loaded first."
                    )

        # Create cases
        created_count = 0
        for case_data in DUMMY_API_CASES:
            court = court_cache[case_data["court_code"]]

            if Case.objects.filter(
                court=court, file_number=case_data["file_number"]
            ).exists():
                logger.warning(
                    "Skipping duplicate case: %s %s",
                    case_data["court_code"],
                    case_data["file_number"],
                )
                continue

            Case.objects.create(
                title=case_data["title"],
                slug=case_data["slug"],
                court=court,
                court_raw=f'{{"name":"{case_data["court_code"]}"}}',
                court_chamber=case_data["court_chamber"],
                date=case_data["date"],
                file_number=case_data["file_number"],
                type=case_data["type"],
                ecli=case_data["ecli"],
                content=case_data["content"],
                abstract=case_data.get("abstract"),
                source_url=case_data["source_url"],
                review_status="pending",
                created_by_token=token,
            )
            created_count += 1
            logger.info("Created API case: %s", case_data["slug"])

        logger.info("Loaded %d dummy API cases (review_status=pending)", created_count)



"""Topic from book title

- Gesetz (über|zum|zur) xxx
    - e.g. Gesetz über Europäische Betriebsräte
    - xx length < 30

- Ortsbewegliche-Druckgeräte-Verordnung
- Weinverordnung
- (Zweite|Dritte|...)Verordnung ((über das|die|das|den)|zur|zum|...)
- Raumordnungsgesetz
- Baugesetzbuch

"""

from oldp.apps.laws.models import Law
from oldp.apps.laws.processing.processing_steps import LawProcessingStep


class ExtractTopics(LawProcessingStep):
    def __init__(self):
        super(ExtractTopics, self).__init__()

    def process(self, law: Law) -> Law:
        # text, refs = self.extract_refs(law.text, law.book.code)
        # TODO LawBook processing step?

        return law



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-07 09:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("courts", "0004_court_updated"),
    ]

    operations = [
        migrations.AddField(
            model_name="court",
            name="wikipedia_title",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name="court",
            name="description",
            field=models.TextField(default=""),
        ),
    ]



from django.conf import settings as django_settings
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from oldp.api.mixins import ReviewStatusFieldMixin
from oldp.apps.cases.models import Case
from oldp.apps.cases.search_indexes import CaseIndex
from oldp.apps.courts.serializers import CourtMinimalSerializer
from oldp.apps.search.api import SearchResultSerializer

CASE_API_FIELDS = (
    "id",
    "slug",
    "court",
    "file_number",
    "date",
    "created_date",
    "updated_date",
    "type",
    "ecli",
    "content",
    "review_status",
)


class CaseSerializer(ReviewStatusFieldMixin, serializers.ModelSerializer):
    court = CourtMinimalSerializer(many=False, read_only=True)
    slug = serializers.ReadOnlyField()

    class Meta:
        model = Case
        fields = CASE_API_FIELDS

        lookup_field = "slug"


class CaseSearchSerializer(SearchResultSerializer):
    """Serializer for case search results."""

    class Meta:
        fields = [
            "slug",
            "date",
            "text",
            "court",
            "court_jurisdiction",
            "court_level_of_appeal",
            "decision_type",
        ]
        index_classes = [CaseIndex]


class SourceInputSerializer(serializers.Serializer):
    """Nested serializer for source information in case creation.

    If a source with the given name exists, it is reused. Otherwise a new source
    is created with the provided name and homepage.
    """

    name = serializers.CharField(help_text="Source name for lookup or creation")
    homepage = serializers.URLField(
        required=False,
        default="",
        allow_blank=True,
        help_text="Source homepage URL (used only when creating a new source)",
    )


class CaseCreateSerializer(serializers.Serializer):
    """Serializer for creating cases via API.

    Accepts court_name instead of court FK, with automatic resolution.
    Validates inputs based on CASE_CREATION_VALIDATION settings.
    """

    # Required fields
    court_name = serializers.CharField(
        help_text="Court name for automatic resolution (e.g., 'Bundesgerichtshof', 'AG Berlin')"
    )
    file_number = serializers.CharField(
        help_text="Court file number (e.g., 'I ZR 123/21')"
    )
    date = serializers.DateField(help_text="Publication date (YYYY-MM-DD format)")
    content = serializers.CharField(help_text="Full case content in HTML format")

    # Optional fields
    type = serializers.CharField(
        required=False,
        allow_blank=True,
        help_text="Type of decision (e.g., 'Urteil', 'Beschluss')",
    )
    ecli = serializers.CharField(
        required=False, allow_blank=True, help_text="European Case Law Identifier"
    )
    abstract = serializers.CharField(
        required=False,
        allow_blank=True,
        help_text="Case summary/abstract in HTML format",
    )
    title = serializers.CharField(
        required=False, allow_blank=True, help_text="Case title"
    )
    source = SourceInputSerializer(
        required=False,
        help_text="Source information (name, homepage). If omitted, the default source is used.",
    )

    def _get_validation_settings(self):
        """Get validation settings with defaults."""
        defaults = {
            "content_min_length": 10,
            "content_max_length": 10000000,
            "file_number_min_length": 1,
            "file_number_max_length": 100,
            "title_max_length": 255,
            "abstract_max_length": 50000,
            "court_name_max_length": 255,
        }
        settings = getattr(django_settings, "CASE_CREATION_VALIDATION", {})
        return {**defaults, **settings}

    def validate_court_name(self, value):
        """Validate court_name field."""
        settings = self._get_validation_settings()
        max_length = settings.get("court_name_max_length", 255)

        if not value or not value.strip():
            raise serializers.ValidationError(_("Court name cannot be empty."))

        if len(value) > max_length:
            raise serializers.ValidationError(
                _("Court name must not exceed %(max_length)s characters.")
                % {"max_length": max_length}
            )

        return value.strip()

    def validate_file_number(self, value):
        """Validate file_number field."""
        settings = self._get_validation_settings()
        min_length = settings.get("file_number_min_length", 1)
        max_length = settings.get("file_number_max_length", 100)

        if not value or not value.strip():
            raise serializers.ValidationError(_("File number cannot be empty."))

        value = value.strip()
        if len(value) < min_length:
            raise serializers.ValidationError(
                _("File number must be at least %(min_length)s characters.")
                % {"min_length": min_length}
            )
        if len(value) > max_length:
            raise serializers.ValidationError(
                _("File number must not exceed %(max_length)s characters.")
                % {"max_length": max_length}
            )

        return value

    def validate_content(self, value):
        """Validate content field."""
        settings = self._get_validation_settings()
        min_length = settings.get("content_min_length", 10)
        max_length = settings.get("content_max_length", 10000000)

        if not value:
            raise serializers.ValidationError(_("Content cannot be empty."))

        if len(value) < min_length:
            raise serializers.ValidationError(
                _("Content must be at least %(min_length)s characters.")
                % {"min_length": min_length}
            )
        if len(value) > max_length:
            raise serializers.ValidationError(
                _("Content must not exceed %(max_length)s characters.")
                % {"max_length": max_length}
            )

        return value

    def validate_title(self, value):
        """Validate title field."""
        if not value:
            return value

        settings = self._get_validation_settings()
        max_length = settings.get("title_max_length", 255)

        if len(value) > max_length:
            raise serializers.ValidationError(
                _("Title must not exceed %(max_length)s characters.")
                % {"max_length": max_length}
            )

        return value

    def validate_abstract(self, value):
        """Validate abstract field."""
        if not value:
            return value

        settings = self._get_validation_settings()
        max_length = settings.get("abstract_max_length", 50000)

        if len(value) > max_length:
            raise serializers.ValidationError(
                _("Abstract must not exceed %(max_length)s characters.")
                % {"max_length": max_length}
            )

        return value

    def validate_source(self, value):
        """Validate source field."""
        if not value:
            return value

        name = value.get("name", "")
        if not name or not name.strip():
            raise serializers.ValidationError(
                {"name": _("Source name cannot be empty.")}
            )

        if len(name) > 100:
            raise serializers.ValidationError(
                {"name": _("Source name must not exceed 100 characters.")}
            )

        value["name"] = name.strip()
        return value


class CaseUpdateSerializer(serializers.ModelSerializer):
    """Serializer for updating cases via PATCH. Only review_status is writable."""

    class Meta:
        model = Case
        fields = ("review_status",)

    def validate_review_status(self, value):
        if value not in ("pending", "accepted", "rejected"):
            raise serializers.ValidationError(
                f"Invalid review_status: {value}. Must be pending, accepted, or rejected."
            )
        return value


class CaseCreateResponseSerializer(serializers.Serializer):
    """Serializer for case creation response."""

    id = serializers.IntegerField(help_text="Case ID")
    slug = serializers.CharField(help_text="Case slug for URL")
    review_status = serializers.CharField(help_text="Review status of the case")



import logging
import os

from django.test import TestCase, tag

logger = logging.getLogger(__name__)
RESOURCE_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources")


@tag("models")
class SearchModelsTestCase(TestCase):
    """Do not forget to put DJANGO_TEST_WITH_ES to true"""

    fixtures = ["search/courts.json", "search/cases.json"]

    def setUp(self):
        pass

    def tearDown(self):
        # CourtsTestCase.tear_down_courts()
        pass

    # @es_test
    # def test_index(self):
    #     obj = Case.objects.get(pk=1)
    #
    #     self.assertTrue(obj.index(), 'Index operation did not return created=true')



import logging

from oldp.apps.courts.models import Court
from oldp.apps.courts.processing import CourtProcessingStep

logger = logging.getLogger(__name__)


class ProcessingStep(CourtProcessingStep):
    description = "Set review_status=accepted"

    def process(self, court: Court):
        court.review_status = "accepted"

        return court



import logging

from django.conf import settings
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _

from oldp.apps.cases.models import Case
from oldp.apps.laws.models import LawBook
from oldp.utils.cache_per_user import cache_per_role

logger = logging.getLogger(__name__)


@cache_per_role(settings.CACHE_TTL)
def index_view(request):
    law_books = LawBook.objects.filter(latest=True).order_by("-order")
    cases = list(
        Case.get_queryset(request)
        .defer(*Case.defer_fields_list_view)
        .select_related("court")
        .order_by("-updated_date")[:10]
    )

    return render(
        request,
        "homepage/index.html",
        {
            "title": _("Free Access to Legal Data"),
            "nav": "homepage",
            "law_books": law_books,
            "cases": cases,
        },
    )


def error500_view(request, exception=None):
    return render(
        request,
        "errors/500.html",
        {"title": _("Error") + " 500", "exception": exception},
        status=500,
    )


def error404_view(request, exception=None):
    return render(
        request,
        "errors/404.html",
        {"title": "%s - %s" % (_("Error"), _("Not found")), "exception": exception},
        status=404,
    )


def error_permission_denied_view(request, exception=None):
    return render(
        request,
        "errors/permission_denied.html",
        {
            "title": "%s - %s" % (_("Error"), _("Permission denied")),
            "exception": exception,
        },
        status=401,
    )


def error_bad_request_view(request, exception=None):
    return render(
        request,
        "errors/bad_request.html",
        {"title": "%s - %s" % (_("Error"), _("Bad request")), "exception": exception},
        status=400,
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-20 17:07
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("cases", "0001_initial"),
        ("laws", "0002_remove_law_foo"),
    ]

    operations = [
        migrations.CreateModel(
            name="CaseReference",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("to", models.TextField()),
                (
                    "case",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="cases.Case",
                    ),
                ),
                (
                    "law",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="laws.Law",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="CaseReferenceMarker",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
                ("uuid", models.CharField(max_length=36)),
                ("start", models.IntegerField(default=0)),
                ("end", models.IntegerField(default=0)),
                ("line", models.CharField(blank=True, max_length=200)),
                (
                    "referenced_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="cases.Case"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="LawReference",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("to", models.TextField()),
                (
                    "case",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="cases.Case",
                    ),
                ),
                (
                    "law",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="laws.Law",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="LawReferenceMarker",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
                ("uuid", models.CharField(max_length=36)),
                ("start", models.IntegerField(default=0)),
                ("end", models.IntegerField(default=0)),
                ("line", models.CharField(blank=True, max_length=200)),
                (
                    "referenced_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="laws.Law"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="lawreference",
            name="marker",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="references.LawReferenceMarker",
            ),
        ),
        migrations.AddField(
            model_name="casereference",
            name="marker",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="references.CaseReferenceMarker",
            ),
        ),
    ]



from django.contrib import admin

from oldp.apps.processing.admin import ProcessingStepActionsAdmin

from .models import CaseReferenceMarker, LawReferenceMarker, Reference


@admin.register(Reference)
class ReferenceAdmin(ProcessingStepActionsAdmin):
    autocomplete_fields = ["case", "law"]


@admin.register(LawReferenceMarker)
class LawReferenceMarkerAdmin(admin.ModelAdmin):
    autocomplete_fields = ["referenced_by"]


@admin.register(CaseReferenceMarker)
class CaseReferenceMarkerAdmin(admin.ModelAdmin):
    autocomplete_fields = ["referenced_by"]



from oldp.apps.laws.models import Law, LawBook
from oldp.apps.processing.processing_steps import BaseProcessingStep


class LawProcessingStep(BaseProcessingStep):
    description = "Law processing step without description"

    def __init__(self):
        super().__init__()

    def process(self, law: Law) -> Law:
        raise NotImplementedError(
            "Abstract processing step needs to implement this method."
        )


class LawBookProcessingStep(BaseProcessingStep):
    description = "Law book processing step without description"

    def __init__(self):
        super().__init__()

    def process(self, law_book: LawBook) -> LawBook:
        raise NotImplementedError(
            "Abstract processing step needs to implement this method."
        )



# Generated by Django 2.1.2 on 2018-10-24 15:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0012_auto_20180921_1104"),
    ]

    operations = [
        migrations.AlterField(
            model_name="lawbook",
            name="latest",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Is true if this is the latest revision of this book",
            ),
        ),
    ]



from django.conf import settings
from django.core.mail import send_mail
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from oldp.apps.contact.forms import ContactForm, ReportContentForm


def form_view(request):
    subject_tpl = "Contact: %(name)s"
    message_tpl = "Name: %(name)s\nEmail: %(email)s\nMessage:\n\n%(message)s"

    if request.method == "POST":
        contact_form = ContactForm(request.POST)

        if contact_form.is_valid():
            subject = subject_tpl % contact_form.cleaned_data
            message = message_tpl % contact_form.cleaned_data

            send_mail(
                subject, message, settings.DEFAULT_FROM_EMAIL, [settings.SITE_EMAIL]
            )
            return redirect(reverse("contact:thankyou"))
    else:
        contact_form = ContactForm()

    return render(
        request, "contact/form.html", {"title": _("Contact"), "form": contact_form}
    )


def report_content_view(request):
    subject_tpl = "Reported content: %(name)s"
    message_tpl = "Subject: %(subject)s\nSource: %(source)s\nName: %(name)s\nEmail: %(email)s\nMessage:\n\n%(message)s"

    if request.method == "POST":
        form = ReportContentForm(request.POST)

        if form.is_valid():
            subject = subject_tpl % form.cleaned_data
            message = message_tpl % form.cleaned_data

            send_mail(
                subject, message, settings.DEFAULT_FROM_EMAIL, [settings.SITE_EMAIL]
            )
            return redirect(reverse("contact:thankyou"))
    else:
        form = ReportContentForm(initial={"source": request.GET.get("source")})

    return render(
        request,
        "contact/report_content.html",
        {"title": _("Report content"), "form": form},
    )


def thankyou_view(request):
    return render(
        request,
        "contact/thankyou.html",
        {
            "title": _("Contact"),
        },
    )



from datetime import timedelta

from django.contrib.auth.models import User
from django.test import TestCase
from django.utils import timezone
from rest_framework import exceptions

from oldp.apps.accounts.authentication import APITokenAuthentication
from oldp.apps.accounts.models import APIToken


class APITokenAuthenticationTestCase(TestCase):
    """Test cases for APITokenAuthentication backend"""

    def setUp(self):
        self.user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )
        self.auth = APITokenAuthentication()

    def test_authenticate_valid_token(self):
        """Test authentication with a valid token"""
        token = APIToken.objects.create(user=self.user, name="Test Token")

        user, auth_token = self.auth.authenticate_credentials(token.key)

        self.assertEqual(user, self.user)
        self.assertEqual(auth_token, token)

    def test_authenticate_invalid_token(self):
        """Test that authentication fails with invalid token"""
        with self.assertRaises(exceptions.AuthenticationFailed) as context:
            self.auth.authenticate_credentials("invalid_token_key_1234567890123456")

        self.assertIn("Invalid token", str(context.exception))

    def test_authenticate_inactive_token(self):
        """Test that authentication fails with inactive token"""
        token = APIToken.objects.create(
            user=self.user, name="Inactive Token", is_active=False
        )

        with self.assertRaises(exceptions.AuthenticationFailed) as context:
            self.auth.authenticate_credentials(token.key)

        self.assertIn("inactive", str(context.exception).lower())

    def test_authenticate_expired_token(self):
        """Test that authentication fails with expired token"""
        past = timezone.now() - timedelta(days=1)
        token = APIToken.objects.create(
            user=self.user, name="Expired Token", expires_at=past
        )

        with self.assertRaises(exceptions.AuthenticationFailed) as context:
            self.auth.authenticate_credentials(token.key)

        self.assertIn("expired", str(context.exception).lower())

    def test_authenticate_inactive_user(self):
        """Test that authentication fails when user is inactive"""
        token = APIToken.objects.create(user=self.user, name="Token")

        # Deactivate the user
        self.user.is_active = False
        self.user.save()

        with self.assertRaises(exceptions.AuthenticationFailed) as context:
            self.auth.authenticate_credentials(token.key)

        self.assertIn("User inactive", str(context.exception))

    def test_authenticate_updates_last_used(self):
        """Test that successful authentication updates last_used timestamp"""
        token = APIToken.objects.create(user=self.user, name="Token")
        self.assertIsNone(token.last_used)

        # Authenticate
        self.auth.authenticate_credentials(token.key)

        # Refresh token from database
        token.refresh_from_db()

        # Verify last_used was updated
        self.assertIsNotNone(token.last_used)
        time_diff = timezone.now() - token.last_used
        self.assertLess(time_diff.total_seconds(), 5)

    def test_authenticate_valid_token_not_expired(self):
        """Test authentication with valid token that has future expiration"""
        future = timezone.now() + timedelta(days=30)
        token = APIToken.objects.create(
            user=self.user, name="Future Token", expires_at=future
        )

        user, auth_token = self.auth.authenticate_credentials(token.key)

        self.assertEqual(user, self.user)
        self.assertEqual(auth_token, token)

    def test_authenticate_with_scopes(self):
        """Test that authentication works with scoped tokens"""
        token = APIToken.objects.create(
            user=self.user, name="Scoped Token", scopes=["read", "write"]
        )

        user, auth_token = self.auth.authenticate_credentials(token.key)

        self.assertEqual(user, self.user)
        self.assertEqual(auth_token, token)
        self.assertTrue(auth_token.has_scope("read"))
        self.assertTrue(auth_token.has_scope("write"))
        self.assertFalse(auth_token.has_scope("admin"))

    def test_authentication_model(self):
        """Test that authentication uses correct model"""
        self.assertEqual(self.auth.model, APIToken)

    def test_authentication_keyword(self):
        """Test that authentication uses correct keyword"""
        self.assertEqual(self.auth.keyword, "Token")



import json
import logging
import os
from unittest import skip

from django.contrib.auth.models import AnonymousUser, User
from django.core.exceptions import ValidationError
from django.test import RequestFactory, TransactionTestCase, tag

from oldp.apps.annotations.models import AnnotationLabel, CaseAnnotation, CaseMarker
from oldp.apps.cases.models import Case

logger = logging.getLogger(__name__)
RESOURCE_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), "resources")


@tag("models")
class AnnotationsModelsTestCase(TransactionTestCase):
    fixtures = [
        "users/with_password_unittest.json",  # password=unittest
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
        "cases/cases.json",
        "annotations/labels.json",
    ]

    def setUp(self):
        self.factory = RequestFactory()

    def test_get_annotations(self):
        c = Case.objects.get(pk=1)
        AnnotationLabel.objects.get(pk=1)

        CaseAnnotation(label_id=1, belongs_to=c, value_str="fff").save()
        CaseAnnotation(label_id=2, belongs_to=c, value_str="fff").save()

        # Check on validations
        try:
            ca2 = CaseAnnotation(label_id=1, belongs_to=c, value_str="fffxxx")
            ca2.clean()
            raise ValueError("Not raised validation error")
        except ValidationError:
            pass

        anos = c.get_annotations()

        self.assertTrue(anos.count(), 2)

    def test_get_annotations_with_request(self):
        c = Case.objects.get(pk=1)

        CaseAnnotation(label_id=1, belongs_to=c, value_str="fff").save()
        CaseAnnotation(label_id=2, belongs_to=c, value_str="fff").save()

        request = self.factory.get(c.get_absolute_url())
        request.user = AnonymousUser()

        guest_annotations = c.get_annotations(request)

        owner_request = self.factory.get(c.get_absolute_url())
        owner_request.user = User.objects.get(pk=1)

        owner_annotations = c.get_annotations(owner_request)

        # One label is private, thus guest should see less annotations
        self.assertEqual(len(guest_annotations), 1)
        self.assertEqual(len(owner_annotations), 2)

    def test_grouped_annotations(self):
        c = Case.objects.get(pk=1)

        CaseAnnotation(label_id=2, belongs_to=c, value_str="foo").save()
        CaseAnnotation(label_id=3, belongs_to=c, value_str="a").save()
        CaseAnnotation(label_id=3, belongs_to=c, value_str="b").save()
        CaseAnnotation(label_id=3, belongs_to=c, value_str="c").save()

        labels = c.get_annotation_labels()

        self.assertEqual(len(labels), 2, "Invalid number of labels")
        self.assertTrue("user/private-label" in labels, "Missing label")
        self.assertTrue("user/private-label-many-values" in labels, "Missing label")
        self.assertTrue(len(labels["user/private-label-many-values"].annotations), 3)

    def test_overlapping_markers(self):
        c = Case.objects.get(pk=1)
        l = AnnotationLabel(name="foo", owner_id=1, many_annotations_per_label=True)
        l.save()

        # Valid markers
        for m in [
            CaseMarker(label=l, belongs_to=c, value_str="A", start=0, end=10),
            CaseMarker(label=l, belongs_to=c, value_str="B", start=10, end=10),
            CaseMarker(label=l, belongs_to=c, value_str="C", start=20, end=25),
        ]:
            m.clean()
            m.save()

        # Try invalid markers
        with self.assertRaises(ValidationError):
            m = CaseMarker(
                label=l, belongs_to=c, value_str="invalid start>end", start=40, end=35
            )
            m.clean()

        with self.assertRaises(ValidationError):
            m = CaseMarker(
                label=l, belongs_to=c, value_str="overlaps with A,B,C", start=5, end=35
            )
            m.clean()

        with self.assertRaises(ValidationError):
            m = CaseMarker(
                label=l, belongs_to=c, value_str="overlaps with B", start=10, end=11
            )
            m.clean()

    @skip
    def test_html_selector(self):
        import lxml.html
        from lxml.cssselect import CSSSelector

        with open(
            os.path.join(RESOURCE_DIR, "../templates/annotations/test.html")
        ) as f:
            print()
            html_str = f.read()

            tree = lxml.html.fromstring(html_str)
            # json_str = '{"start":{"selector":"div>p:nth-of-type(2)","textNodeIndex":0,"offset":0},"end":{"selector":"div>p:nth-of-type(2)","textNodeIndex":0,"offset":35}}'
            # json_str = '{"start":{"selector":"div>p:nth-of-type(2)","textNodeIndex":2,"offset":1},"end":{"selector":"div>p:nth-of-type(2)","textNodeIndex":2,"offset":28}}'
            json_str = '{"start":{"selector":"div>h2:nth-of-type(2)","textNodeIndex":0,"offset":4},"end":{"selector":"div>h2:nth-of-type(2)","textNodeIndex":0,"offset":10}}'
            json_str = '{"start":{"selector":"div>p:nth-of-type(2)","textNodeIndex":2,"offset":1},"end":{"selector":"div>p:nth-of-type(2)","textNodeIndex":2,"offset":6}}'
            x = json.loads(json_str)

            start = x["start"]
            end = x["end"]
            selected_text = None

            if start["selector"] == end["selector"]:
                start_matches = CSSSelector(start["selector"])(tree)

                if len(start_matches) == 1:
                    if start["textNodeIndex"] == 0 and end["textNodeIndex"] == 0:
                        selected_text = start_matches[0].text[
                            start["offset"] : end["offset"]
                        ]

                        # replace text with {annotation}text{annotation}
                    else:
                        print("textNodeIndex != 0")
                        start_matches = CSSSelector(start["selector"])(tree)

                        for child in start_matches[0].getchildren():
                            print("Text: %s " % child.text)
                            print("Tail: %s" % child.tail)

                            selected_text = child.tail[start["offset"] : end["offset"]]

                elif len(start_matches) > 1:
                    raise ValueError("Multiple start matches found")
                else:
                    raise ValueError("Start node does not exist")

                # for elem in :
                #     print(lxml.html.tostring(elem))
                #     print('Text: %s' % elem.text)
                #     print('Tail: %s' % elem.tail)
                #
                #     res = elem.text[x['start']]
                #
                #     print('---\nChildren')
                #
                #
                #     for c in elem.getchildren():
                #         print('Text: %s ' % c.text)
                #         print('Tail: %s' % c.tail)

            else:
                raise ValueError("start != end selector")

            print("Selected text: %s" % selected_text)

            # Processed tree -> HTML
            # Extract annotation markers with positions

            # for elem in CSSSelector(x['end']['selector'])(tree):
            #     print(lxml.html.tostring(elem))



"""Unit tests for the Law and LawBook Creation API.

Tests cover:
- Successful law book creation
- Revision/latest flag management
- Duplicate detection
- Law creation within books
- Book resolution from code
- API token tracking
- Authentication and permissions
"""

from datetime import date

from django.contrib.auth import get_user_model
from django.test import TestCase
from rest_framework import status
from rest_framework.test import APIClient, APITestCase

from oldp.apps.accounts.models import (
    APIToken,
    APITokenPermission,
    APITokenPermissionGroup,
)
from oldp.apps.laws.exceptions import (
    DuplicateLawBookError,
    DuplicateLawError,
    LawBookNotFoundError,
)
from oldp.apps.laws.models import Law, LawBook
from oldp.apps.laws.services import LawBookCreator, LawCreator

User = get_user_model()


class LawBookCreatorTestCase(TestCase):
    """Tests for the LawBookCreator service."""

    def setUp(self):
        self.creator = LawBookCreator()

    def test_create_lawbook_success(self):
        """Test successful law book creation."""
        lawbook = self.creator.create_lawbook(
            code="TESTBUCH",
            title="Testgesetzbuch",
            revision_date=date(2021, 1, 1),
        )

        self.assertIsNotNone(lawbook.pk)
        self.assertEqual(lawbook.code, "TESTBUCH")
        self.assertEqual(lawbook.title, "Testgesetzbuch")
        self.assertEqual(lawbook.slug, "testbuch")
        self.assertTrue(lawbook.latest)

    def test_create_lawbook_duplicate_raises_error(self):
        """Test that creating duplicate law book raises DuplicateLawBookError."""
        # Create initial law book
        self.creator.create_lawbook(
            code="DUPBOOK",
            title="Duplicate Book",
            revision_date=date(2021, 1, 1),
        )

        # Try to create duplicate
        with self.assertRaises(DuplicateLawBookError):
            self.creator.create_lawbook(
                code="DUPBOOK",
                title="Duplicate Book 2",
                revision_date=date(2021, 1, 1),  # Same revision date
            )

    def test_create_newer_revision_becomes_latest(self):
        """Test that creating a newer revision becomes the latest."""
        # Create older revision
        old_book = self.creator.create_lawbook(
            code="REVBOOK",
            title="Revision Book Old",
            revision_date=date(2020, 1, 1),
        )
        self.assertTrue(old_book.latest)

        # Create newer revision
        new_book = self.creator.create_lawbook(
            code="REVBOOK",
            title="Revision Book New",
            revision_date=date(2021, 1, 1),
        )

        # Refresh old book from database
        old_book.refresh_from_db()

        # New book should be latest
        self.assertTrue(new_book.latest)
        # Old book should no longer be latest
        self.assertFalse(old_book.latest)

    def test_create_older_revision_not_latest(self):
        """Test that creating an older revision does not become latest."""
        # Create newer revision first
        new_book = self.creator.create_lawbook(
            code="OLDREVBOOK",
            title="Older Revision Book New",
            revision_date=date(2021, 1, 1),
        )
        self.assertTrue(new_book.latest)

        # Create older revision
        old_book = self.creator.create_lawbook(
            code="OLDREVBOOK",
            title="Older Revision Book Old",
            revision_date=date(2020, 1, 1),
        )

        # Refresh new book from database
        new_book.refresh_from_db()

        # Old book should not be latest
        self.assertFalse(old_book.latest)
        # New book should still be latest
        self.assertTrue(new_book.latest)

    def test_create_lawbook_with_api_token_tracking(self):
        """Test that API token is tracked on created law book."""
        user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )
        token = APIToken.objects.create(user=user, name="Test Token")

        lawbook = self.creator.create_lawbook(
            code="TOKENBOOK",
            title="Token Book",
            revision_date=date(2021, 1, 1),
            api_token=token,
        )

        self.assertEqual(lawbook.created_by_token, token)


class LawCreatorTestCase(TestCase):
    """Tests for the LawCreator service."""

    def setUp(self):
        self.creator = LawCreator()
        # Create a law book for testing
        self.lawbook = LawBook.objects.create(
            code="TESTLAW",
            title="Test Law Book",
            slug="testlaw",
            revision_date=date(2021, 1, 1),
            latest=True,
        )

    def test_create_law_success(self):
        """Test successful law creation."""
        law = self.creator.create_law(
            book_code="TESTLAW",
            section="§ 1",
            title="Test Section",
            content="<p>Test content</p>",
        )

        self.assertIsNotNone(law.pk)
        self.assertEqual(law.book, self.lawbook)
        self.assertEqual(law.section, "§ 1")
        self.assertEqual(law.title, "Test Section")
        self.assertEqual(law.slug, "1")  # Auto-generated from section

    def test_create_law_with_custom_slug(self):
        """Test law creation with custom slug."""
        law = self.creator.create_law(
            book_code="TESTLAW",
            section="§ 2",
            title="Custom Slug Section",
            content="<p>Test content</p>",
            slug="custom-slug",
        )

        self.assertEqual(law.slug, "custom-slug")

    def test_create_law_duplicate_raises_error(self):
        """Test that creating duplicate law raises DuplicateLawError."""
        # Create initial law
        self.creator.create_law(
            book_code="TESTLAW",
            section="§ 99",
            title="Original Law",
            content="<p>Original content</p>",
        )

        # Try to create duplicate (same book and slug)
        with self.assertRaises(DuplicateLawError):
            self.creator.create_law(
                book_code="TESTLAW",
                section="§ 99",  # Same section, generates same slug
                title="Duplicate Law",
                content="<p>Duplicate content</p>",
            )

    def test_create_law_book_not_found_raises_error(self):
        """Test that non-existent book raises LawBookNotFoundError."""
        with self.assertRaises(LawBookNotFoundError):
            self.creator.create_law(
                book_code="NONEXISTENT",
                section="§ 1",
                title="Test",
                content="<p>Test</p>",
            )

    def test_create_law_with_specific_revision(self):
        """Test law creation with specific book revision."""
        # Create an older revision
        old_book = LawBook.objects.create(
            code="TESTLAW",
            title="Test Law Book Old",
            slug="testlaw",
            revision_date=date(2020, 1, 1),
            latest=False,
        )

        # Create law in the old revision
        law = self.creator.create_law(
            book_code="TESTLAW",
            section="§ 100",
            title="Old Revision Law",
            content="<p>Old content</p>",
            revision_date=date(2020, 1, 1),
        )

        self.assertEqual(law.book, old_book)

    def test_create_law_with_api_token_tracking(self):
        """Test that API token is tracked on created law."""
        user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )
        token = APIToken.objects.create(user=user, name="Test Token")

        law = self.creator.create_law(
            book_code="TESTLAW",
            section="§ 200",
            title="Token Law",
            content="<p>Token content</p>",
            api_token=token,
        )

        self.assertEqual(law.created_by_token, token)

    def test_resolve_lawbook_by_code_latest(self):
        """Test resolving law book by code (uses latest)."""
        book = self.creator.resolve_lawbook("TESTLAW")
        self.assertEqual(book, self.lawbook)

    def test_resolve_lawbook_by_code_and_date(self):
        """Test resolving law book by code and specific date."""
        book = self.creator.resolve_lawbook("TESTLAW", revision_date=date(2021, 1, 1))
        self.assertEqual(book, self.lawbook)


class LawBookCreationAPITestCase(APITestCase):
    """Integration tests for the LawBook Creation API endpoint."""

    def setUp(self):
        # Create user
        self.user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )

        # Create permission and permission group for write access
        self.write_permission, _ = APITokenPermission.objects.get_or_create(
            resource="lawbooks", action="write"
        )
        self.permission_group = APITokenPermissionGroup.objects.create(
            name="lawbooks_write_group"
        )
        self.permission_group.permissions.add(self.write_permission)

        # Create API token with write permission
        self.token = APIToken.objects.create(
            user=self.user,
            name="Test Token",
            permission_group=self.permission_group,
        )

        self.client = APIClient()
        self.client.force_authenticate(user=self.user, token=self.token)

    def test_create_lawbook_success(self):
        """Test successful law book creation via API."""
        data = {
            "code": "APIBOOK",
            "title": "API Created Book",
            "revision_date": "2021-05-15",
        }

        response = self.client.post("/api/law_books/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertIn("id", response.data)
        self.assertIn("slug", response.data)
        self.assertIn("latest", response.data)
        self.assertTrue(response.data["latest"])

    def test_create_lawbook_duplicate_returns_409(self):
        """Test duplicate law book returns 409 Conflict."""
        # Create first law book
        LawBook.objects.create(
            code="DUPAPI",
            title="Duplicate API Book",
            slug="dupapi",
            revision_date=date(2021, 1, 1),
        )

        data = {
            "code": "DUPAPI",
            "title": "Duplicate API Book 2",
            "revision_date": "2021-01-01",
        }

        response = self.client.post("/api/law_books/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_409_CONFLICT)

    def test_create_lawbook_without_authentication_returns_401(self):
        """Test unauthenticated request returns 401."""
        self.client.force_authenticate(user=None, token=None)

        data = {
            "code": "NOAUTH",
            "title": "No Auth Book",
            "revision_date": "2021-01-01",
        }

        response = self.client.post("/api/law_books/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_create_lawbook_tracks_api_token(self):
        """Test that API token is tracked on created law book."""
        data = {
            "code": "TOKENAPI",
            "title": "Token API Book",
            "revision_date": "2021-05-15",
        }

        response = self.client.post("/api/law_books/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        # Verify token was tracked
        lawbook = LawBook.objects.get(pk=response.data["id"])
        self.assertEqual(lawbook.created_by_token, self.token)


class LawCreationAPITestCase(APITestCase):
    """Integration tests for the Law Creation API endpoint."""

    def setUp(self):
        # Create user
        self.user = User.objects.create_user(
            username="testuser", email="test@example.com", password="testpass123"
        )

        # Create permission and permission group for write access
        self.write_permission, _ = APITokenPermission.objects.get_or_create(
            resource="laws", action="write"
        )
        self.permission_group = APITokenPermissionGroup.objects.create(
            name="laws_write_group"
        )
        self.permission_group.permissions.add(self.write_permission)

        # Create API token with write permission
        self.token = APIToken.objects.create(
            user=self.user,
            name="Test Token",
            permission_group=self.permission_group,
        )

        self.client = APIClient()
        self.client.force_authenticate(user=self.user, token=self.token)

        # Create a law book for testing
        self.lawbook = LawBook.objects.create(
            code="APILAW",
            title="API Law Book",
            slug="apilaw",
            revision_date=date(2021, 1, 1),
            latest=True,
        )

    def test_create_law_success(self):
        """Test successful law creation via API."""
        data = {
            "book_code": "APILAW",
            "section": "§ 1",
            "title": "API Created Law",
            "content": "<p>API law content</p>",
        }

        response = self.client.post("/api/laws/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertIn("id", response.data)
        self.assertIn("slug", response.data)
        self.assertIn("book_id", response.data)
        self.assertEqual(response.data["book_id"], self.lawbook.id)

    def test_create_law_book_not_found_returns_400(self):
        """Test law creation with non-existent book returns 400."""
        data = {
            "book_code": "NONEXISTENT",
            "section": "§ 1",
            "title": "Test Law",
            "content": "<p>Test content</p>",
        }

        response = self.client.post("/api/laws/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_law_duplicate_returns_409(self):
        """Test duplicate law returns 409 Conflict."""
        # Create first law
        Law.objects.create(
            book=self.lawbook,
            section="§ 999",
            title="Original Law",
            content="<p>Original</p>",
            slug="999",
        )

        data = {
            "book_code": "APILAW",
            "section": "§ 999",  # Same section, same slug
            "title": "Duplicate Law",
            "content": "<p>Duplicate</p>",
        }

        response = self.client.post("/api/laws/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_409_CONFLICT)

    def test_create_law_without_authentication_returns_401(self):
        """Test unauthenticated request returns 401."""
        self.client.force_authenticate(user=None, token=None)

        data = {
            "book_code": "APILAW",
            "section": "§ 1",
            "title": "No Auth Law",
            "content": "<p>No auth content</p>",
        }

        response = self.client.post("/api/laws/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_create_law_tracks_api_token(self):
        """Test that API token is tracked on created law."""
        data = {
            "book_code": "APILAW",
            "section": "§ 500",
            "title": "Token Law",
            "content": "<p>Token content</p>",
        }

        response = self.client.post("/api/laws/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        # Verify token was tracked
        law = Law.objects.get(pk=response.data["id"])
        self.assertEqual(law.created_by_token, self.token)

    def test_create_law_missing_required_fields_returns_400(self):
        """Test missing required fields returns 400."""
        data = {
            "book_code": "APILAW",
            # Missing section, title, content
        }

        response = self.client.post("/api/laws/", data, format="json")

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn("section", response.data)
        self.assertIn("title", response.data)
        self.assertIn("content", response.data)


class LawBookRevisionIntegrationTestCase(APITestCase):
    """Full integration tests for law book revision management."""

    def setUp(self):
        self.user = User.objects.create_user(
            username="revisionuser", email="revision@example.com", password="testpass"
        )

        write_permission, _ = APITokenPermission.objects.get_or_create(
            resource="lawbooks", action="write"
        )
        permission_group = APITokenPermissionGroup.objects.create(
            name="revision_write_group"
        )
        permission_group.permissions.add(write_permission)

        self.token = APIToken.objects.create(
            user=self.user, name="Revision Token", permission_group=permission_group
        )

        self.client = APIClient()
        self.client.force_authenticate(user=self.user, token=self.token)

    def test_revision_management_flow(self):
        """Test complete revision management flow."""
        # Create initial revision (2020)
        response1 = self.client.post(
            "/api/law_books/",
            {
                "code": "REVTEST",
                "title": "Revision Test Book 2020",
                "revision_date": "2020-01-01",
            },
            format="json",
        )
        self.assertEqual(response1.status_code, status.HTTP_201_CREATED)
        self.assertTrue(response1.data["latest"])
        book_2020_id = response1.data["id"]

        # Create newer revision (2021) - should become latest
        response2 = self.client.post(
            "/api/law_books/",
            {
                "code": "REVTEST",
                "title": "Revision Test Book 2021",
                "revision_date": "2021-01-01",
            },
            format="json",
        )
        self.assertEqual(response2.status_code, status.HTTP_201_CREATED)
        self.assertTrue(response2.data["latest"])

        # Verify 2020 revision is no longer latest
        book_2020 = LawBook.objects.get(pk=book_2020_id)
        self.assertFalse(book_2020.latest)

        # Create older revision (2019) - should NOT become latest
        response3 = self.client.post(
            "/api/law_books/",
            {
                "code": "REVTEST",
                "title": "Revision Test Book 2019",
                "revision_date": "2019-01-01",
            },
            format="json",
        )
        self.assertEqual(response3.status_code, status.HTTP_201_CREATED)
        self.assertFalse(response3.data["latest"])

        # Verify counts
        self.assertEqual(LawBook.objects.filter(code="REVTEST").count(), 3)
        self.assertEqual(LawBook.objects.filter(code="REVTEST", latest=True).count(), 1)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-02-09 17:48
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0003_auto_20180209_0926"),
    ]

    operations = [
        migrations.AlterField(
            model_name="case",
            name="court",
            field=models.ForeignKey(
                default=1,
                help_text="Responsible court entity",
                on_delete=django.db.models.deletion.CASCADE,
                to="courts.Court",
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="court_chamber",
            field=models.CharField(
                blank=True,
                help_text="Court chamber (e.g. 1. Senat)",
                max_length=150,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="court_raw",
            field=models.CharField(
                default="{}",
                help_text="Raw court information from scraper (JSON)",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="date",
            field=models.DateField(
                db_index=True, help_text="Publication date as in source", null=True
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="file_number",
            field=models.CharField(
                blank=True,
                help_text="File number as defined by court",
                max_length=100,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="from_juris",
            field=models.BooleanField(
                db_index=True, default=False, help_text="If content is from juris"
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="pdf_url",
            field=models.URLField(
                blank=True,
                help_text="URL to original PDF file (not in use)",
                max_length=255,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="raw",
            field=models.TextField(
                blank=True, help_text="Raw content from scraper", null=True
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="sections",
            field=models.TextField(
                default="[]", help_text="JSON field of CaseSection objects"
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="slug",
            field=models.SlugField(
                help_text="Used to urls (consists of court, date, file number)",
                max_length=200,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_homepage",
            field=models.URLField(help_text="Link to source homepage"),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_name",
            field=models.CharField(
                help_text="Name of source (scraper class)", max_length=100
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_url",
            field=models.URLField(
                help_text="Path to source of scraper", max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="text",
            field=models.TextField(help_text="Plain text for searching", null=True),
        ),
        migrations.AlterField(
            model_name="case",
            name="title",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Title (currently not used due to copyright issues)",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="type",
            field=models.CharField(
                blank=True,
                help_text="Type of decision (Urteil, Beschluss, ...)",
                max_length=100,
                null=True,
            ),
        ),
    ]



from django.test import tag
from django.urls import reverse

from oldp.apps.lib.tests import ExtendedLiveServerTestCase


@tag("views")
class CourtsViewsTestCase(ExtendedLiveServerTestCase):
    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def test_index(self):
        res = self.client.get(reverse("courts:index"))

        self.assertContains(res, "Amtsgericht Aalen")
        self.assertContains(res, "EuGH")
        self.assertContains(res, "Unknown state")

    def test_index_filter(self):
        res = self.client.get(reverse("courts:index") + "?state=1")

        self.assertNotContains(res, "Amtsgericht Aalen")
        self.assertContains(res, "EuGH")
        self.assertContains(res, "Unknown court")

        self.assertStringOrder(res, "EuGH", "Unknown court")

        # With reverse order
        res = self.client.get(reverse("courts:index") + "?o=-name")
        self.assertStringOrder(res, "Unknown court", "EuGH")

    def test_detail(self):
        res = self.client.get(reverse("courts:detail", args=("ag-aalen",)))

        self.assertContains(res, "Amtsgericht Aalen")
        self.assertContains(res, "AGAALEN")

        # TODO test for cases



# Generated by Django 2.1.7 on 2019-03-15 12:03

import re

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("annotations", "0003_casemarker"),
    ]

    operations = [
        migrations.AlterField(
            model_name="annotationlabel",
            name="color",
            field=models.CharField(
                default="#CCCCCC",
                max_length=18,
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$"),
                        "Enter a valid color.",
                        "invalid",
                    )
                ],
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-01-03 13:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0006_auto_20180103_1158"),
    ]

    operations = [
        migrations.AddField(
            model_name="lawbook",
            name="latest",
            field=models.BooleanField(default=True),
        ),
    ]



"""Custom admin views for law and lawbook statistics dashboards."""

from oldp.apps.laws.models import Law, LawBook
from oldp.apps.processing.admin_views import CreationDashboardView


class LawCreationDashboardView(CreationDashboardView):
    """Dashboard showing law creation statistics with filtering and batch processing."""

    model = Law
    admin_changelist_url_name = "admin:laws_law_changelist"
    app_label = "laws"
    entity_name = "Law"
    entity_name_plural = "Laws"


class LawBookCreationDashboardView(CreationDashboardView):
    """Dashboard showing law book creation statistics with filtering and batch processing."""

    model = LawBook
    admin_changelist_url_name = "admin:laws_lawbook_changelist"
    app_label = "laws"
    entity_name = "Law Book"
    entity_name_plural = "Law Books"



# Generated by Django 2.0.4 on 2018-04-24 08:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0004_auto_20180209_1748"),
    ]

    operations = [
        migrations.AlterField(
            model_name="case",
            name="court",
            field=models.ForeignKey(
                default=1,
                help_text="Responsible court entity",
                on_delete=django.db.models.deletion.SET_DEFAULT,
                to="courts.Court",
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="court_raw",
            field=models.CharField(
                default="{}",
                help_text="Raw court information from crawler (JSON)",
                max_length=255,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="raw",
            field=models.TextField(
                blank=True, help_text="Raw content from crawler", null=True
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_name",
            field=models.CharField(
                help_text="Name of source (crawler class)", max_length=100
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="source_url",
            field=models.URLField(
                help_text="Path to source of crawler", max_length=255
            ),
        ),
    ]



"""Tests for profiling toggles (django-silk and django-querycount)."""

import unittest
from importlib.util import find_spec

from django.conf import settings
from django.test import TestCase, override_settings


class ProfilingSettingsTestCase(TestCase):
    """Test that profiling toggles are off by default and don't leak into normal config."""

    def test_profiling_disabled_by_default(self):
        """PROFILING_ENABLED defaults to False; silk not in INSTALLED_APPS or MIDDLEWARE."""
        self.assertFalse(settings.PROFILING_ENABLED)
        self.assertNotIn("silk", settings.INSTALLED_APPS)
        self.assertNotIn("silk.middleware.SilkyMiddleware", settings.MIDDLEWARE)

    def test_querycount_disabled_by_default(self):
        """QUERYCOUNT_ENABLED defaults to False; querycount not in MIDDLEWARE."""
        self.assertFalse(settings.QUERYCOUNT_ENABLED)
        self.assertNotIn(
            "querycount.middleware.QueryCountMiddleware", settings.MIDDLEWARE
        )

    def test_silk_url_not_registered_when_disabled(self):
        """When profiling is off, /silk/ returns 404."""
        response = self.client.get("/silk/")
        self.assertEqual(response.status_code, 404)


@unittest.skipUnless(find_spec("querycount"), "django-querycount not installed")
@override_settings(
    MIDDLEWARE=list(settings.MIDDLEWARE)
    + ["querycount.middleware.QueryCountMiddleware"],
    QUERYCOUNT={
        "THRESHOLDS": {
            "MEDIUM": 50,
            "HIGH": 200,
            "MIN_TIME_TO_LOG": 0,
            "MIN_QUERY_COUNT_TO_LOG": 0,
        },
        "DISPLAY_DUPLICATES": 5,
    },
    STORAGES={
        "staticfiles": {
            "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
        }
    },
)
class QuerycountMiddlewareTestCase(TestCase):
    """Test that querycount middleware works when enabled via settings override."""

    def test_querycount_middleware_runs_without_error(self):
        """The middleware integrates with Django and responds successfully."""
        response = self.client.get("/")
        self.assertIn(response.status_code, [200, 301, 302])



"""Services for case management."""

from oldp.apps.cases.services.case_creator import CaseCreator
from oldp.apps.cases.services.court_resolver import CourtResolver

__all__ = ["CourtResolver", "CaseCreator"]



# Generated by Django 5.1.15 on 2026-02-25 11:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0003_create_default_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="apitoken",
            name="rate_limit",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Custom rate limit in requests per hour for this token. Leave blank to use the default rate.",
                null=True,
                verbose_name="Rate Limit",
            ),
        ),
    ]



from django.contrib.sitemaps import GenericSitemap

from oldp.apps.cases.models import Case


class CaseSitemap(GenericSitemap):
    def __init__(self):
        super().__init__(
            {
                "queryset": Case.get_queryset()
                .select_related("court")
                .defer(*Case.defer_fields_list_view)
                .order_by("-updated_date"),
                "date_field": "updated_date",
            },
            priority=1.0,
            changefreq="daily",
        )



"""Version utilities for OLDP package."""

import importlib.metadata


def get_version() -> str:
    """Get the version of the oldp package dynamically.

    This function tries multiple methods to get the version:
    1. From package metadata (works when installed)
    2. From oldp.__version__ (fallback)

    Returns:
        str: The version string of the oldp package

    Raises:
        RuntimeError: If version cannot be determined
    """
    try:
        # Try to get version from installed package metadata
        return importlib.metadata.version("oldp")
    except importlib.metadata.PackageNotFoundError:
        # Fallback to __version__ in __init__.py
        try:
            from oldp import __version__

            return __version__
        except ImportError:
            raise RuntimeError("Could not determine oldp package version")


def get_version_info() -> dict:
    """Get detailed version information.

    Returns:
        dict: Dictionary containing version and metadata
    """
    version = get_version()

    try:
        metadata = importlib.metadata.metadata("oldp")
        return {
            "version": version,
            "name": metadata.get("Name", "oldp"),
            "summary": metadata.get("Summary", ""),
            "author": metadata.get("Author", ""),
            "license": metadata.get("License", ""),
        }
    except importlib.metadata.PackageNotFoundError:
        return {
            "version": version,
            "name": "oldp",
            "summary": "Open Legal Data Platform",
            "author": "Malte Ostendorff",
            "license": "MIT",
        }



"""Tests for /me/ API endpoints.

Tests cover:
- List filtering by authenticated token
- Isolation between different users/tokens
- review_status visibility (always shown for owner)
- Authentication required
"""

from datetime import date

from django.contrib.auth import get_user_model
from django.test import override_settings
from rest_framework import status
from rest_framework.test import APIClient, APITestCase

from oldp.apps.accounts.models import (
    APIToken,
    APITokenPermission,
    APITokenPermissionGroup,
)
from oldp.apps.cases.models import Case
from oldp.apps.courts.models import Court, State
from oldp.apps.laws.models import Law, LawBook

User = get_user_model()


@override_settings(
    CACHES={"default": {"BACKEND": "django.core.cache.backends.dummy.DummyCache"}}
)
class MeEndpointsTestCase(APITestCase):
    """Tests for /me/cases/, /me/law_books/, /me/laws/, /me/courts/ endpoints."""

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def setUp(self):
        self.court = Court.objects.exclude(pk=Court.DEFAULT_ID).first()

        # Permission group with read access
        read_perm, _ = APITokenPermission.objects.get_or_create(
            resource="cases", action="read"
        )
        group = APITokenPermissionGroup.objects.create(name="read_group")
        group.permissions.add(read_perm)

        # User A with token
        self.user_a = User.objects.create_user(
            username="user_a", email="a@example.com", password="pass"
        )
        self.token_a = APIToken.objects.create(
            user=self.user_a, name="Token A", permission_group=group
        )

        # User B with token
        self.user_b = User.objects.create_user(
            username="user_b", email="b@example.com", password="pass"
        )
        self.token_b = APIToken.objects.create(
            user=self.user_b, name="Token B", permission_group=group
        )

        # Cases
        self.case_a = Case.objects.create(
            court=self.court,
            file_number="ME-CASE-A-001",
            date=date(2021, 1, 1),
            content="<p>Case by token A</p>",
            review_status="pending",
            created_by_token=self.token_a,
        )
        self.case_b = Case.objects.create(
            court=self.court,
            file_number="ME-CASE-B-001",
            date=date(2021, 2, 1),
            content="<p>Case by token B</p>",
            review_status="pending",
            created_by_token=self.token_b,
        )
        self.case_no_token = Case.objects.create(
            court=self.court,
            file_number="ME-CASE-NOTOKEN",
            date=date(2021, 3, 1),
            content="<p>Case without token</p>",
            review_status="accepted",
        )

        # Law books
        self.lawbook_a = LawBook.objects.create(
            code="METEST-A",
            slug="metest-a",
            title="Test Book A",
            revision_date=date(2021, 1, 1),
            latest=True,
            review_status="pending",
            created_by_token=self.token_a,
        )
        self.lawbook_b = LawBook.objects.create(
            code="METEST-B",
            slug="metest-b",
            title="Test Book B",
            revision_date=date(2021, 1, 1),
            latest=True,
            review_status="pending",
            created_by_token=self.token_b,
        )

        # Laws
        self.law_a = Law.objects.create(
            book=self.lawbook_a,
            title="Test Law A",
            section="1",
            content="<p>Law by A</p>",
            slug="metest-a-1",
            review_status="pending",
            created_by_token=self.token_a,
        )
        self.law_b = Law.objects.create(
            book=self.lawbook_b,
            title="Test Law B",
            section="1",
            content="<p>Law by B</p>",
            slug="metest-b-1",
            review_status="pending",
            created_by_token=self.token_b,
        )

        # Courts created by tokens
        state = State.objects.first()
        self.court_a = Court.objects.create(
            name="Me Test Court A",
            code="METESTA",
            slug="me-test-court-a",
            state=state,
            review_status="pending",
            created_by_token=self.token_a,
        )
        self.court_b = Court.objects.create(
            name="Me Test Court B",
            code="METESTB",
            slug="me-test-court-b",
            state=state,
            review_status="pending",
            created_by_token=self.token_b,
        )

        self.client = APIClient()

    # ── Cases ──────────────────────────────────────────────────────

    def test_me_cases_list_shows_own_items(self):
        """Token A sees only cases created by token A."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/cases/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.case_a.pk, ids)
        self.assertNotIn(self.case_b.pk, ids)
        self.assertNotIn(self.case_no_token.pk, ids)

    def test_me_cases_list_excludes_others_items(self):
        """Token B does not see token A's cases."""
        self.client.force_authenticate(user=self.user_b, token=self.token_b)
        response = self.client.get("/api/me/cases/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.case_b.pk, ids)
        self.assertNotIn(self.case_a.pk, ids)

    def test_me_cases_review_status_visible(self):
        """review_status is always visible for the owner's items."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/cases/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        for item in response.data["results"]:
            self.assertIn("review_status", item)

    # ── Law Books ──────────────────────────────────────────────────

    def test_me_law_books_list_shows_own_items(self):
        """Token A sees only law books created by token A."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/law_books/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.lawbook_a.pk, ids)
        self.assertNotIn(self.lawbook_b.pk, ids)

    def test_me_law_books_list_excludes_others_items(self):
        """Token B does not see token A's law books."""
        self.client.force_authenticate(user=self.user_b, token=self.token_b)
        response = self.client.get("/api/me/law_books/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.lawbook_b.pk, ids)
        self.assertNotIn(self.lawbook_a.pk, ids)

    def test_me_law_books_review_status_visible(self):
        """review_status is visible for owner's law books."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/law_books/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        for item in response.data["results"]:
            self.assertIn("review_status", item)

    # ── Laws ───────────────────────────────────────────────────────

    def test_me_laws_list_shows_own_items(self):
        """Token A sees only laws created by token A."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/laws/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.law_a.pk, ids)
        self.assertNotIn(self.law_b.pk, ids)

    def test_me_laws_list_excludes_others_items(self):
        """Token B does not see token A's laws."""
        self.client.force_authenticate(user=self.user_b, token=self.token_b)
        response = self.client.get("/api/me/laws/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.law_b.pk, ids)
        self.assertNotIn(self.law_a.pk, ids)

    def test_me_laws_review_status_visible(self):
        """review_status is visible for owner's laws."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/laws/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        for item in response.data["results"]:
            self.assertIn("review_status", item)

    # ── Courts ─────────────────────────────────────────────────────

    def test_me_courts_list_shows_own_items(self):
        """Token A sees only courts created by token A."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/courts/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.court_a.pk, ids)
        self.assertNotIn(self.court_b.pk, ids)

    def test_me_courts_list_excludes_others_items(self):
        """Token B does not see token A's courts."""
        self.client.force_authenticate(user=self.user_b, token=self.token_b)
        response = self.client.get("/api/me/courts/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        ids = {item["id"] for item in response.data["results"]}
        self.assertIn(self.court_b.pk, ids)
        self.assertNotIn(self.court_a.pk, ids)

    def test_me_courts_review_status_visible(self):
        """review_status is visible for owner's courts."""
        self.client.force_authenticate(user=self.user_a, token=self.token_a)
        response = self.client.get("/api/me/courts/")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        for item in response.data["results"]:
            self.assertIn("review_status", item)

    # ── Authentication ─────────────────────────────────────────────

    def test_me_cases_requires_auth(self):
        """Unauthenticated request returns 401."""
        response = self.client.get("/api/me/cases/")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_me_law_books_requires_auth(self):
        """Unauthenticated request returns 401."""
        response = self.client.get("/api/me/law_books/")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_me_laws_requires_auth(self):
        """Unauthenticated request returns 401."""
        response = self.client.get("/api/me/laws/")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_me_courts_requires_auth(self):
        """Unauthenticated request returns 401."""
        response = self.client.get("/api/me/courts/")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)



from django.db import models


class RelatedContent(models.Model):
    seed_content = None
    related_content = None
    score = models.DecimalField(max_digits=12, decimal_places=8)

    class Meta:
        abstract = True

    def set_relation(self, seed_id, related_id):
        self.seed_content_id = seed_id
        self.related_content_id = related_id

    def set_score(self, score):
        self.score = score

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return "RelatedContent(seed=%s, related=%s, score=%s)" % (
            self.seed_content,
            self.related_content,
            self.score,
        )


class SearchableContent(object):
    """Abstract class for Elasticsearch content (using now haystack instead)"""

    def get_title(self):
        raise NotImplementedError("SearchableContent needs to implement get_title()")

    def get_es_url(self):
        return None  # modelresult/cases.case.123


class SearchQuery(models.Model):
    query = models.CharField(
        max_length=200,
        unique=True,
        help_text="Query as entered by user but all lower case",
    )
    counter = models.IntegerField(default=1, help_text="Count of query executions")
    created_date = models.DateTimeField(
        auto_now_add=True, help_text="First query execution"
    )
    updated_date = models.DateTimeField(auto_now=True, help_text="Last query execution")

    class Meta:
        indexes = [
            models.Index(fields=["query"]),
        ]

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return "SearchQuery(%s, %i, %s)" % (self.query, self.counter, self.updated_date)



# Generated by Django 5.1.15 on 2026-02-06 15:55

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0021_add_performance_indexes"),
    ]

    operations = [
        migrations.AddField(
            model_name="lawbook",
            name="created_date",
            field=models.DateTimeField(
                auto_now_add=True,
                db_index=True,
                default=django.utils.timezone.now,
                help_text="Entry is created at this date time",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="lawbook",
            name="updated_date",
            field=models.DateTimeField(
                auto_now=True, db_index=True, help_text="Date time of last change"
            ),
        ),
    ]



from urllib.parse import quote_plus

from django import template
from django.conf import settings
from django.urls import reverse

register = template.Library()


@register.simple_tag(takes_context=True)
def report_content_url(context):
    """Get URL to report content form with the current URL attached to it"""
    request = context["request"]

    # We use only the path here, ignoring all GET parameters
    source_url = settings.SITE_URL + request.path

    return reverse("contact:report_content") + "?source=" + quote_plus(source_url)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-01-03 11:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("laws", "0004_auto_20171222_1911"),
    ]

    operations = [
        migrations.AlterField(
            model_name="lawbook",
            name="code",
            field=models.CharField(max_length=100, null=True),
        ),
    ]



# Generated by Django 5.1.15 on 2026-01-29 16:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0003_create_default_permissions"),
        ("cases", "0023_alter_case_following_cases_alter_case_id_and_more"),
        ("courts", "0018_alter_city_id_alter_country_id_alter_court_id_and_more"),
        ("sources", "0002_alter_source_id"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="case",
            index=models.Index(fields=["ecli"], name="cases_case_ecli_idx"),
        ),
        migrations.AddIndex(
            model_name="case",
            index=models.Index(
                fields=["file_number"], name="cases_case_file_number_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="case",
            index=models.Index(
                fields=["court", "-date"], name="cases_case_court_date_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="relatedcase",
            index=models.Index(
                fields=["seed_content", "-score"], name="cases_relcase_seed_score_idx"
            ),
        ),
    ]



"""Law creator service for creating laws within law books."""

import logging
from typing import Optional

from django.utils.text import slugify

from oldp.apps.laws.exceptions import DuplicateLawError, LawBookNotFoundError
from oldp.apps.laws.models import Law, LawBook

logger = logging.getLogger(__name__)


class LawCreator:
    """Service for creating laws within law books.

    This service handles:
    - Law book resolution from code
    - Duplicate checking
    - API token tracking
    """

    def resolve_lawbook(
        self,
        book_code: str,
        revision_date=None,
        use_latest: bool = True,
    ) -> LawBook:
        """Resolve a law book from code and optional revision date.

        Args:
            book_code: Law book code (e.g., "BGB", "StGB")
            revision_date: Optional specific revision date
            use_latest: If True and no revision_date, use the latest revision

        Returns:
            LawBook instance

        Raises:
            LawBookNotFoundError: If law book cannot be found
        """
        if not book_code:
            raise LawBookNotFoundError("Book code is required")

        if revision_date:
            qs = LawBook.objects.filter(code=book_code, revision_date=revision_date)
        elif use_latest:
            qs = LawBook.objects.filter(code=book_code, latest=True)
        else:
            raise LawBookNotFoundError(
                "Either revision_date or use_latest=True is required"
            )

        book = qs.order_by("-pk").first()
        if book is None:
            if revision_date:
                raise LawBookNotFoundError(
                    f"Law book with code '{book_code}' and revision date '{revision_date}' not found"
                )
            else:
                raise LawBookNotFoundError(
                    f"No latest revision found for law book code '{book_code}'"
                )
        return book

    def check_duplicate(self, book: LawBook, slug: str) -> bool:
        """Check if a law with the same book and slug already exists.

        Args:
            book: LawBook instance
            slug: Law slug

        Returns:
            True if duplicate exists, False otherwise
        """
        return Law.objects.filter(book=book, slug=slug).exists()

    def create_law(
        self,
        book_code: str,
        section: str,
        title: str,
        content: str,
        revision_date=None,
        slug: Optional[str] = None,
        order: int = 0,
        amtabk: Optional[str] = None,
        kurzue: Optional[str] = None,
        doknr: Optional[str] = None,
        footnotes: Optional[str] = None,
        api_token=None,
    ) -> Law:
        """Create a new law within a law book.

        Args:
            book_code: Law book code (e.g., "BGB", "StGB")
            section: Section identifier (e.g., "§ 1", "Art. 1")
            title: Verbose title of the law
            content: Law content in HTML
            revision_date: Optional specific book revision date (uses latest if not specified)
            slug: Optional slug (auto-generated from section if not provided)
            order: Order within the book
            amtabk: Official abbreviation
            kurzue: Short title
            doknr: Document number from XML source
            footnotes: Footnotes as JSON array
            api_token: APIToken used for creation (for tracking)

        Returns:
            Created Law instance

        Raises:
            DuplicateLawError: If law with same book+slug exists
            LawBookNotFoundError: If law book cannot be found
        """
        # Resolve the law book
        book = self.resolve_lawbook(book_code, revision_date)

        # Generate slug from section if not provided
        if not slug:
            slug = slugify(section) if section else "untitled"

        # Check for duplicates
        if self.check_duplicate(book, slug):
            raise DuplicateLawError(
                f"A law with book '{book.code}' (revision {book.revision_date}) "
                f"and slug '{slug}' already exists."
            )

        # Create the law
        law = Law(
            book=book,
            section=section,
            title=title,
            content=content,
            slug=slug,
            order=order,
            amtabk=amtabk or "",
            kurzue=kurzue or "",
            doknr=doknr or "",
            footnotes=footnotes or "",
        )

        # Track the API token if provided
        if api_token is not None:
            from oldp.apps.accounts.models import APIToken

            if isinstance(api_token, APIToken):
                law.created_by_token = api_token
                law.review_status = "pending"

        law.save()

        logger.info(
            "Created law %s/%s (id=%s, section=%s)",
            book.code,
            slug,
            law.pk,
            section,
        )

        return law



from django.conf import settings
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page
from django.views.decorators.vary import vary_on_cookie, vary_on_headers
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import status, viewsets
from rest_framework.generics import GenericAPIView
from rest_framework.mixins import ListModelMixin
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework.viewsets import ViewSetMixin

from oldp.api import SmallResultsSetPagination
from oldp.api.mixins import ReviewStatusFilterMixin
from oldp.apps.accounts.permissions import HasTokenPermission
from oldp.apps.laws.models import Law, LawBook
from oldp.apps.laws.search_indexes import LawIndex
from oldp.apps.laws.serializers import (
    LawBookCreateSerializer,
    LawBookSerializer,
    LawCreateSerializer,
    LawSearchSerializer,
    LawSerializer,
)
from oldp.apps.laws.services import LawBookCreator, LawCreator
from oldp.apps.search.api import SearchFilter, SearchViewMixin
from oldp.apps.search.filters import SearchSchemaFilter


class LawViewSet(ReviewStatusFilterMixin, viewsets.ModelViewSet):
    permission_classes = [HasTokenPermission]
    token_resource = "laws"

    queryset = Law.objects.all().order_by("order")
    serializer_class = LawSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = ("book_id", "book__latest", "book__revision_date")

    def get_permissions(self):
        """Return permissions based on action - require auth for write operations."""
        action = getattr(self, "action", None)
        if action in ["create", "update", "partial_update", "destroy"]:
            return [IsAuthenticated(), HasTokenPermission()]
        return [HasTokenPermission()]

    def get_serializer_class(self):
        """Return appropriate serializer based on action."""
        if getattr(self, "action", None) == "create":
            return LawCreateSerializer
        return LawSerializer

    @method_decorator(cache_page(settings.CACHE_TTL))
    @method_decorator(vary_on_headers("Authorization", "Accept-Language", "Host"))
    @method_decorator(vary_on_cookie)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get_queryset(self):
        return super().get_queryset().select_related("book", "created_by_token")

    def create(self, request, *args, **kwargs):
        """Create a new law within a law book.

        The law book is resolved from book_code (uses latest revision by default).
        """
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        data = serializer.validated_data

        # Get the API token from request
        api_token = getattr(request, "auth", None)

        # Create the law
        creator = LawCreator()
        law = creator.create_law(
            book_code=data["book_code"],
            section=data["section"],
            title=data["title"],
            content=data["content"],
            revision_date=data.get("revision_date"),
            slug=data.get("slug"),
            order=data.get("order", 0),
            amtabk=data.get("amtabk"),
            kurzue=data.get("kurzue"),
            doknr=data.get("doknr"),
            footnotes=data.get("footnotes"),
            api_token=api_token,
        )

        # Return minimal response
        response_data = {
            "id": law.id,
            "slug": law.slug,
            "book_id": law.book_id,
            "review_status": law.review_status,
        }

        return Response(response_data, status=status.HTTP_201_CREATED)


class LawBookViewSet(ReviewStatusFilterMixin, viewsets.ModelViewSet):
    permission_classes = [HasTokenPermission]
    token_resource = "lawbooks"

    queryset = LawBook.objects.all().order_by("code")
    serializer_class = LawBookSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = ("slug", "code", "latest", "revision_date")

    def get_permissions(self):
        """Return permissions based on action - require auth for write operations."""
        action = getattr(self, "action", None)
        if action in ["create", "update", "partial_update", "destroy"]:
            return [IsAuthenticated(), HasTokenPermission()]
        return [HasTokenPermission()]

    def get_serializer_class(self):
        """Return appropriate serializer based on action."""
        if getattr(self, "action", None) == "create":
            return LawBookCreateSerializer
        return LawBookSerializer

    @method_decorator(cache_page(settings.CACHE_TTL))
    @method_decorator(vary_on_headers("Authorization", "Accept-Language", "Host"))
    @method_decorator(vary_on_cookie)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)

    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .select_related("created_by_token")
            .defer("changelog", "footnotes", "sections")
        )

    def create(self, request, *args, **kwargs):
        """Create a new law book.

        If this revision is newer than existing revisions for the same code,
        it automatically becomes the 'latest' revision.
        """
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        data = serializer.validated_data

        # Get the API token from request
        api_token = getattr(request, "auth", None)

        # Create the law book
        creator = LawBookCreator()
        lawbook = creator.create_lawbook(
            code=data["code"],
            title=data["title"],
            revision_date=data["revision_date"],
            order=data.get("order", 0),
            changelog=data.get("changelog"),
            footnotes=data.get("footnotes"),
            sections=data.get("sections"),
            api_token=api_token,
        )

        # Return minimal response
        response_data = {
            "id": lawbook.id,
            "slug": lawbook.slug,
            "latest": lawbook.latest,
            "review_status": lawbook.review_status,
        }

        return Response(response_data, status=status.HTTP_201_CREATED)


class LawSearchSchemaFilter(SearchSchemaFilter):
    search_index_class = LawIndex

    def get_default_schema_operation_parameters(self):
        return [
            {
                "name": "text",
                "required": True,
                "in": "query",
                "description": "Search query on text content (Lucene syntax support).",
                "schema": {"type": "string"},
            }
        ]


class LawSearchViewSet(SearchViewMixin, ListModelMixin, ViewSetMixin, GenericAPIView):
    """Search view"""

    permission_classes = (AllowAny,)
    pagination_class = SmallResultsSetPagination  # limit page (other content field blows up response size)
    search_models = [Law]
    serializer_class = LawSearchSerializer
    filter_backends = (
        SearchFilter,
        LawSearchSchemaFilter,
    )

    @method_decorator(cache_page(settings.CACHE_TTL))
    @method_decorator(vary_on_headers("Authorization", "Accept-Language", "Host"))
    @method_decorator(vary_on_cookie)
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)



# Generated by Django 5.1.15 on 2026-01-29 16:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("accounts", "0003_create_default_permissions"),
        ("laws", "0020_add_created_by_token"),
        ("topics", "0003_alter_topic_id"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="law",
            index=models.Index(fields=["previous"], name="laws_law_previous_idx"),
        ),
        migrations.AddIndex(
            model_name="law",
            index=models.Index(
                fields=["book", "order"], name="laws_law_book_order_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="law",
            index=models.Index(fields=["section"], name="laws_law_section_idx"),
        ),
        migrations.AddIndex(
            model_name="lawbook",
            index=models.Index(
                fields=["code", "latest"], name="laws_lawbook_code_latest_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="relatedlaw",
            index=models.Index(
                fields=["seed_content", "-score"], name="laws_rellaw_seed_score_idx"
            ),
        ),
    ]



# Generated by Django 2.0.4 on 2018-04-30 12:25

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cases", "0008_auto_20180426_0806"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="case",
            name="text",
        ),
        migrations.AlterField(
            model_name="case",
            name="created_date",
            field=models.DateTimeField(
                auto_now_add=True, help_text="Entry is created at this date time"
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="raw",
            field=models.TextField(
                blank=True,
                help_text="Raw content (HTML) from crawler that can used to reconstruct all case information",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="updated_date",
            field=models.DateTimeField(
                auto_now=True, help_text="Date time of last change"
            ),
        ),
    ]



from rest_framework import permissions


class OwnerPrivatePermission(permissions.BasePermission):
    """Write: Only staff or owner
    Read: not private or owner or staff
    """

    def has_permission(self, request, view):
        if (
            request.method not in permissions.SAFE_METHODS
            and not request.user.is_authenticated
        ):
            return False
        else:
            return True

    def _is_public(self, obj):
        """Check if an object is publicly visible.

        Supports models using review_status (Case, Law, ...) and models
        using a private boolean (AnnotationLabel).
        """
        if hasattr(obj, "review_status"):
            return obj.review_status == "accepted"
        if hasattr(obj, "private"):
            return not obj.private
        return True

    def has_object_permission(self, request, view, obj):
        # Read requests
        if request.method in permissions.SAFE_METHODS:
            if request.user.is_authenticated:
                if request.user.is_staff:
                    return True
                else:
                    return self._is_public(obj) or obj.get_owner() == request.user
            else:
                return self._is_public(obj)
        else:
            # Write requests
            if request.user.is_authenticated:
                if request.user.is_staff:
                    return True
                else:
                    return obj.get_owner() == request.user
            else:
                return False



import binascii
import os

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class APITokenPermission(models.Model):
    """Individual permission for API tokens.

    Defines granular access control for specific resources and actions.
    Format: <resource>:<action> (e.g., "cases:read", "laws:write")
    """

    RESOURCE_CHOICES = [
        ("cases", _("Cases")),
        ("laws", _("Laws")),
        ("courts", _("Courts")),
        ("lawbooks", _("Law Books")),
        ("references", _("References")),
        ("annotations", _("Annotations")),
    ]

    ACTION_CHOICES = [
        ("read", _("Read")),
        ("write", _("Write")),
        ("delete", _("Delete")),
    ]

    resource = models.CharField(
        _("Resource"),
        max_length=50,
        choices=RESOURCE_CHOICES,
        help_text=_("The resource this permission applies to"),
    )
    action = models.CharField(
        _("Action"),
        max_length=20,
        choices=ACTION_CHOICES,
        help_text=_("The action allowed on this resource"),
    )
    description = models.TextField(
        _("Description"),
        blank=True,
        help_text=_("Optional description of what this permission allows"),
    )

    class Meta:
        verbose_name = _("API Token Permission")
        verbose_name_plural = _("API Token Permissions")
        unique_together = [["resource", "action"]]
        ordering = ["resource", "action"]

    def __str__(self):
        return f"{self.resource}:{self.action}"

    def get_permission_string(self):
        """Get the permission as a string in format resource:action"""
        return f"{self.resource}:{self.action}"


class APITokenPermissionGroup(models.Model):
    """Group of permissions that can be assigned to API tokens.

    This allows administrators to create reusable permission sets
    (e.g., "read_only", "full_access", "default").
    """

    name = models.CharField(
        _("Name"),
        max_length=100,
        unique=True,
        help_text=_("Unique name for this permission group"),
    )
    description = models.TextField(
        _("Description"),
        blank=True,
        help_text=_("Description of what this permission group allows"),
    )
    permissions = models.ManyToManyField(
        APITokenPermission,
        related_name="permission_groups",
        verbose_name=_("Permissions"),
        blank=True,
        help_text=_("Permissions included in this group"),
    )
    is_default = models.BooleanField(
        _("Is Default"),
        default=False,
        help_text=_("Whether this is the default permission group for new tokens"),
    )
    created = models.DateTimeField(_("Created"), auto_now_add=True)
    updated = models.DateTimeField(_("Updated"), auto_now=True)

    class Meta:
        verbose_name = _("API Token Permission Group")
        verbose_name_plural = _("API Token Permission Groups")
        ordering = ["name"]

    def __str__(self):
        return self.name

    def has_permission(self, resource, action):
        """Check if this group has a specific permission"""
        return self.permissions.filter(resource=resource, action=action).exists()

    def get_permission_list(self):
        """Get all permissions as a list of strings"""
        return [p.get_permission_string() for p in self.permissions.all()]


class APIToken(models.Model):
    """API Token model that supports multiple tokens per user with enhanced features.

    This model replaces the default DRF Token model with a more feature-rich implementation
    that supports:
    - Multiple tokens per user
    - Named tokens for easy identification
    - Token expiration
    - Usage tracking
    - Active/inactive status
    - Optional scope restrictions
    """

    # Core fields
    key = models.CharField(
        _("Key"),
        max_length=40,
        unique=True,
        db_index=True,
        help_text=_("The API token key"),
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="api_tokens",
        on_delete=models.CASCADE,
        verbose_name=_("User"),
        help_text=_("The user this token belongs to"),
    )
    name = models.CharField(
        _("Name"),
        max_length=100,
        help_text=_(
            "A descriptive name for this token (e.g., 'Production Server', 'CI/CD Pipeline')"
        ),
    )

    # Timestamp fields
    created = models.DateTimeField(
        _("Created"), auto_now_add=True, help_text=_("When this token was created")
    )
    last_used = models.DateTimeField(
        _("Last used"),
        null=True,
        blank=True,
        help_text=_("When this token was last used"),
    )
    expires_at = models.DateTimeField(
        _("Expires at"),
        null=True,
        blank=True,
        help_text=_("When this token expires (null = never expires)"),
    )

    # Status and permissions
    is_active = models.BooleanField(
        _("Active"), default=True, help_text=_("Whether this token is currently active")
    )
    permission_group = models.ForeignKey(
        "APITokenPermissionGroup",
        related_name="tokens",
        on_delete=models.PROTECT,
        null=True,
        blank=True,
        verbose_name=_("Permission Group"),
        help_text=_(
            "The permission group assigned to this token (defines what resources it can access)"
        ),
    )
    scopes = models.JSONField(
        _("Scopes"),
        default=list,
        blank=True,
        help_text=_(
            "Deprecated: Use permission_group instead. List of scopes this token has access to"
        ),
    )
    rate_limit = models.PositiveIntegerField(
        _("Rate Limit"),
        null=True,
        blank=True,
        help_text=_(
            "Custom rate limit in requests per hour for this token. "
            "Leave blank to use the default rate."
        ),
    )

    class Meta:
        verbose_name = _("API Token")
        verbose_name_plural = _("API Tokens")
        ordering = ["-created"]
        indexes = [
            models.Index(fields=["user", "is_active"]),
            models.Index(fields=["expires_at"]),
        ]

    def __str__(self):
        return f"{self.user.username} - {self.name}"

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super().save(*args, **kwargs)

    @classmethod
    def generate_key(cls):
        """Generate a random API key"""
        return binascii.hexlify(os.urandom(20)).decode()

    def is_expired(self):
        """Check if the token has expired"""
        if self.expires_at is None:
            return False
        return timezone.now() > self.expires_at

    def is_valid(self):
        """Check if the token is valid (active and not expired)"""
        return self.is_active and not self.is_expired()

    def has_scope(self, scope):
        """Check if the token has a specific scope (deprecated method).
        Use has_permission() instead.
        """
        if not self.scopes:
            return True  # No scopes means full access
        return scope in self.scopes

    def has_permission(self, resource, action):
        """Check if the token has permission for a specific resource and action.

        Args:
            resource: The resource name (e.g., "cases", "laws")
            action: The action name (e.g., "read", "write", "delete")

        Returns:
            bool: True if the token has the permission, False otherwise
        """
        # If no permission group is assigned, check legacy scopes
        if not self.permission_group:
            # For backward compatibility, check scopes
            if not self.scopes:
                return True  # No restrictions means full access
            permission_string = f"{resource}:{action}"
            return (
                permission_string in self.scopes
                or resource in self.scopes
                or action in self.scopes
            )

        # Check if the permission group has this permission
        return self.permission_group.has_permission(resource, action)

    def get_permissions(self):
        """Get all permissions this token has as a list of strings"""
        if self.permission_group:
            return self.permission_group.get_permission_list()
        return self.scopes if self.scopes else []

    def get_rate_limit(self):
        """Get the custom rate limit for this token, or None for default."""
        return self.rate_limit

    def mark_used(self):
        """Update the last_used timestamp"""
        self.last_used = timezone.now()
        self.save(update_fields=["last_used"])



# Generated by Django 2.1.2 on 2019-01-24 10:26

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("cases", "0017_case_source_file"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="AnnotationLabel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Verbose name, e.g. This Awesome annotation",
                        max_length=100,
                    ),
                ),
                (
                    "slug",
                    models.SlugField(
                        help_text="Identifier, e.g. this-awesome-annotation",
                        max_length=100,
                    ),
                ),
                (
                    "trusted",
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text="Trusted annotations are display by default in front end",
                    ),
                ),
                (
                    "private",
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text="Private annotations are only visible to its author",
                    ),
                ),
                (
                    "many_annotations_per_label",
                    models.BooleanField(
                        default=False,
                        help_text="A content object can have more than one annotation per label",
                    ),
                ),
                (
                    "use_marker",
                    models.BooleanField(
                        default=False,
                        help_text="Marker annotations are extracted from the text content and have a position in the text",
                    ),
                ),
                (
                    "annotation_value_type",
                    models.CharField(
                        choices=[("str", "String"), ("int", "Integer")],
                        default="str",
                        help_text="Annotation values must be in this data type",
                        max_length=5,
                    ),
                ),
                ("color", models.CharField(blank=True, max_length=18, null=True)),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Entry is created at this date time",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="Date time of last change"
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Label",
                "db_table": "annotations_label",
            },
        ),
        migrations.CreateModel(
            name="CaseAnnotation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value_str", models.CharField(blank=True, max_length=250, null=True)),
                ("value_int", models.IntegerField(blank=True, null=True)),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="Entry is created at this date time",
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, help_text="Date time of last change"
                    ),
                ),
                (
                    "belongs_to",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="cases.Case"
                    ),
                ),
                (
                    "label",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="annotations.AnnotationLabel",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AlterUniqueTogether(
            name="annotationlabel",
            unique_together={("slug", "owner")},
        ),
    ]



"""Unit tests for case processing steps.

Tests cover:
- SetReviewPending processing step
- SetReviewAccepted processing step
- AssignCourt processing step (remove_chamber method)
"""

import logging

from django.test import TestCase, tag

from oldp.apps.cases.models import Case
from oldp.apps.cases.processing.processing_steps.assign_court import (
    ProcessingStep as AssignCourtStep,
)
from oldp.apps.cases.processing.processing_steps.set_review_accepted import (
    ProcessingStep as SetReviewAcceptedStep,
)
from oldp.apps.cases.processing.processing_steps.set_review_pending import (
    ProcessingStep as SetReviewPendingStep,
)

logger = logging.getLogger(__name__)


@tag("processing", "cases")
class SetReviewPendingStepTestCase(TestCase):
    """Tests for the SetReviewPending processing step."""

    def test_description(self):
        """Test that the step has correct description."""
        step = SetReviewPendingStep()
        self.assertEqual(step.description, "Set review_status=pending")

    def test_sets_review_status_to_pending(self):
        """Test that process sets review_status to pending."""
        step = SetReviewPendingStep()
        case = Case(review_status="accepted", file_number="TEST/123")

        result = step.process(case)

        self.assertEqual(result.review_status, "pending")

    def test_returns_case_instance(self):
        """Test that process returns the Case instance."""
        step = SetReviewPendingStep()
        case = Case(file_number="TEST/123")

        result = step.process(case)

        self.assertIsInstance(result, Case)
        self.assertIs(result, case)

    def test_already_pending_case(self):
        """Test processing a case that is already pending."""
        step = SetReviewPendingStep()
        case = Case(review_status="pending", file_number="TEST/123")

        result = step.process(case)

        self.assertEqual(result.review_status, "pending")


@tag("processing", "cases")
class SetReviewAcceptedStepTestCase(TestCase):
    """Tests for the SetReviewAccepted processing step."""

    def test_description(self):
        """Test that the step has correct description."""
        step = SetReviewAcceptedStep()
        self.assertEqual(step.description, "Set review_status=accepted")

    def test_sets_review_status_to_accepted(self):
        """Test that process sets review_status to accepted."""
        step = SetReviewAcceptedStep()
        case = Case(review_status="pending", file_number="TEST/123")

        result = step.process(case)

        self.assertEqual(result.review_status, "accepted")

    def test_returns_case_instance(self):
        """Test that process returns the Case instance."""
        step = SetReviewAcceptedStep()
        case = Case(file_number="TEST/123")

        result = step.process(case)

        self.assertIsInstance(result, Case)
        self.assertIs(result, case)

    def test_already_accepted_case(self):
        """Test processing a case that is already accepted."""
        step = SetReviewAcceptedStep()
        case = Case(review_status="accepted", file_number="TEST/123")

        result = step.process(case)

        self.assertEqual(result.review_status, "accepted")


@tag("processing", "cases")
class AssignCourtRemoveChamberTestCase(TestCase):
    """Tests for the AssignCourt processing step's remove_chamber method."""

    def setUp(self):
        self.step = AssignCourtStep()

    def test_remove_chamber_simple_number(self):
        """Test removing numbered chamber from court name."""
        name = "LG Koblenz 14. Zivilkammer"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "LG Koblenz")
        self.assertIsNotNone(chamber)
        self.assertIn("14", chamber)

    def test_remove_chamber_senat(self):
        """Test removing Senat designation from court name."""
        name = "OLG Koblenz 2. Senat für Bußgeldsachen"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "OLG Koblenz")
        self.assertIsNotNone(chamber)

    def test_remove_chamber_kammer_fur(self):
        """Test removing 'Kammer für' designation."""
        name = "LG Kiel Kammer für Handelssachen"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "LG Kiel")
        self.assertIn("Kammer für Handelssachen", chamber)

    def test_remove_chamber_senat_fur(self):
        """Test removing 'Senat für' designation."""
        name = "OLG München Senat für Familiensachen"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "OLG München")
        self.assertIn("Senat für Familiensachen", chamber)

    def test_remove_chamber_kartellsenat(self):
        """Test removing combined chamber name (e.g., Kartellsenat)."""
        name = "Schleswig-Holsteinisches Oberlandesgericht Kartellsenat"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "Schleswig-Holsteinisches Oberlandesgericht")
        self.assertIn("Kartellsenat", chamber)

    def test_remove_chamber_no_chamber(self):
        """Test with court name without chamber designation."""
        name = "Amtsgericht Berlin"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "Amtsgericht Berlin")
        self.assertIsNone(chamber)

    def test_remove_chamber_zivilkammer(self):
        """Test removing combined Zivilkammer."""
        name = "LG Hamburg Zivilkammer"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertEqual(result_name, "LG Hamburg")
        self.assertIn("Zivilkammer", chamber)

    def test_remove_chamber_strips_whitespace(self):
        """Test that result is stripped of extra whitespace."""
        name = "LG Berlin  10. Strafkammer"

        result_name, chamber = self.step.remove_chamber(name)

        self.assertFalse(result_name.endswith(" "))
        self.assertFalse(result_name.startswith(" "))


@tag("processing", "cases")
class AssignCourtStepTestCase(TestCase):
    """Tests for the AssignCourt processing step."""

    fixtures = [
        "locations/countries.json",
        "locations/states.json",
        "locations/cities.json",
        "courts/courts.json",
    ]

    def setUp(self):
        self.step = AssignCourtStep()

    def test_description(self):
        """Test that the step has correct description."""
        self.assertEqual(self.step.description, "Assign court to cases")

    def test_process_uses_court_raw(self):
        """Test that processing uses court_raw field."""
        from oldp.apps.courts.models import Court

        # Create case with court_raw
        case = Case(
            file_number="TEST/123",
            court_raw='{"name": "Amtsgericht Berlin"}',
            court_id=Court.DEFAULT_ID,
        )

        # The step should attempt to process the court_raw
        # Even if court is not found, it shouldn't error
        try:
            result = self.step.process(case)
            # Should return a Case instance
            self.assertIsInstance(result, Case)
        except Court.DoesNotExist:
            # Expected if court isn't in database
            pass



import logging

from django.db.models.base import ModelBase
from haystack.models import SearchResult
from haystack.query import SearchQuerySet

from oldp.apps.laws.models import Law

logger = logging.getLogger(__name__)


class RelatedContentFinder(object):
    model = None
    model_relation = None

    def __init__(self, model: ModelBase, model_relation: ModelBase):
        super(RelatedContentFinder, self).__init__()

        if not isinstance(model, ModelBase):
            raise TypeError("model needs to be django DB model (ModelBase type)")

        if not isinstance(model_relation, ModelBase):
            raise TypeError(
                "model_relation needs to be django DB model (ModelBase type)"
            )

        self.model = model
        self.model_relation = model_relation

    def handle_item(self, item):
        """Perform MLT-query on item"""
        sqs = SearchQuerySet().models(self.model)
        mlt_results: list[SearchResult] = sqs.more_like_this(item)
        saved = []
        for result in mlt_results:
            rel = self.model_relation(score=result.score)
            rel.set_relation(seed_id=item.pk, related_id=result.pk)
            rel.save()

            logger.debug("Saved %s" % rel)

            saved.append(rel)

        return saved

    def handle(self, options):
        items = self.model.objects.filter().order_by("-updated_date")

        if self.model == Law:
            items = items.exclude(book__latest=False)

        if options["limit"] > 0:
            items = items[: options["limit"]]

        if options["empty"]:
            self.model_relation.objects.all().delete()

        if len(items) < 1:
            logger.info("No content available")

        for item in items:
            self.handle_item(item)

        logger.info("done")



import logging

from django.core.management import BaseCommand

from oldp.apps.cases.models import Case, RelatedCase
from oldp.apps.laws.models import Law, RelatedLaw
from oldp.apps.search.processing import RelatedContentFinder

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Assign related cases with MoreLikeThis results"

    def __init__(self):
        super(Command, self).__init__()

    def add_arguments(self, parser):
        parser.add_argument("type", type=str, help="Content type (case, law, ...)")
        parser.add_argument("--limit", type=int, default=20)
        parser.add_argument("--empty", action="store_true", default=False)

    def handle(self, *args, **options):
        # TODO as processing step + admin action

        content_types = {
            "case": {
                "model": Case,
                "relation": RelatedCase,
                # 'es_type': 'case',
                # 'mlt_fields': ['text', 'title']
            },
            "law": {
                "model": Law,
                "relation": RelatedLaw,
                # 'es_type': 'law',
                # 'mlt_fields': ['text', 'title']
            },
        }

        if options["type"] in content_types:
            content_type = content_types[options["type"]]

            logger.info("Generating related content for: %s" % content_type)

            RelatedContentFinder(
                content_type["model"], content_type["relation"]
            ).handle(options)

        else:
            raise ValueError(
                "Provided content type is not supports: %s. Use instead: %s"
                % (options["type"], content_types.keys())
            )



# Generated by Django 5.1.15 on 2026-01-26 18:08

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("annotations", "0004_default_color"),
    ]

    operations = [
        migrations.AlterField(
            model_name="annotationlabel",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="caseannotation",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="casemarker",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from haystack import indexes

from oldp.apps.cases.models import Case


class CaseIndex(indexes.SearchIndex, indexes.Indexable):
    FACET_MODEL_NAME = "Case"

    text = indexes.CharField(document=True, use_template=True)
    title = indexes.CharField()
    absolute_url = indexes.CharField()
    model_type = indexes.CharField()

    review_status = indexes.CharField(model_attr="review_status")

    slug = indexes.CharField(model_attr="slug")

    facet_model_name = indexes.CharField(faceted=True)

    decision_type = indexes.CharField(faceted=True, null=True)
    court = indexes.CharField(faceted=True)
    court_jurisdiction = indexes.CharField(faceted=True, null=True)
    court_level_of_appeal = indexes.CharField(faceted=True, null=True)

    date = indexes.DateField(faceted=True)

    exact_matches = indexes.CharField()  # boost on exact match with this field

    def get_model(self):
        return Case

    def prepare_title(self, obj):
        return obj.get_title()

    def prepare_absolute_url(self, obj):
        return obj.get_absolute_url()

    def prepare_model_type(self, obj):
        return "Case"

    def prepare_facet_model_name(self, obj):
        return self.FACET_MODEL_NAME

    def prepare_decision_type(self, obj):
        return obj.type

    def prepare_court(self, obj):
        return obj.court.code  # TODO short name?

    def prepare_court_jurisdiction(self, obj):
        return obj.court.jurisdiction

    def prepare_court_level_of_appeal(self, obj):
        return obj.court.level_of_appeal

    def prepare_date(self, obj):
        return obj.date  # .strftime('%Y-%m%-%d')

    def index_queryset(self, using=None):
        return Case.get_queryset().select_related("court", "court__state")



from django.utils.text import slugify
from haystack import indexes

from oldp.apps.laws.models import Law


class LawIndex(indexes.SearchIndex, indexes.Indexable):
    """# Define files that will be excluded in JSON export / Elasticsearch document
    es_fields_exclude = ['content', 'amtabk', 'footnotes', 'doknr']
    es_type = 'law'

    """

    FACET_MODEL_NAME = "Law"

    text = indexes.CharField(document=True, use_template=True)
    slug = indexes.CharField(model_attr="slug")
    review_status = indexes.CharField(model_attr="review_status")
    title = indexes.CharField()
    absolute_url = indexes.CharField()
    model_type = indexes.CharField()
    facet_model_name = indexes.CharField(faceted=True)
    book_code = indexes.CharField(faceted=True)

    # title_auto = indexes.EdgeNgramField()
    exact_matches = indexes.CharField()  # boost on exact match with this field

    def get_model(self):
        return Law

    def prepare_title(self, obj):
        return obj.get_title()

    def prepare_absolute_url(self, obj):
        return obj.get_absolute_url()

    def prepare_model_type(self, obj):
        return "Law"

    def prepare_facet_model_name(self, obj):
        return self.FACET_MODEL_NAME

    def prepare_book_code(self, obj):
        return obj.book.code

    def prepare_exact_matches(self, obj):
        """All possible navigational queries"""
        sect = slugify(obj.section)
        code = obj.book.code.lower()

        return [
            code + " " + sect,
            sect + " " + code,
            # no whitespace
            code + sect,
            sect + code,
            obj.title,
        ]

    def index_queryset(self, using=None):
        return (
            self.get_model()
            .objects.filter(review_status="accepted")
            .select_related("book")
            .filter(book__latest=True)
        )



import logging

from django.db import models
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.utils.text import slugify

logger = logging.getLogger(__name__)


class Topic(models.Model):
    title = models.CharField(
        max_length=200,
        help_text="Verbose title of topic",
        unique=True,
    )
    slug = models.SlugField(
        max_length=200,
        help_text="Slug based on title",
        unique=True,
        blank=True,
        db_index=True,
    )
    created_date = models.DateTimeField(
        auto_now_add=True, help_text="Date of creation of this database entry"
    )
    updated_date = models.DateTimeField(
        auto_now=True, help_text="Last change of database entry"
    )

    def __repr__(self):
        return "<Topic({})>".format(self.title)

    def __str__(self):
        return self.title


@receiver(pre_save, sender=Topic)
def pre_save_topic(sender, instance: Topic, *args, **kwargs):
    if instance.slug is None or instance.slug == "":
        instance.slug = slugify(instance.title)


class TopicContent(models.Model):
    """Abstract for content models that can be assigned to topics (e.g. case, law)."""

    topics = models.ManyToManyField(
        Topic,
        help_text="Topics that are covered by this content",
        blank=True,
    )

    class Meta:
        abstract = True
